import * as i0 from '@angular/core';
import { input, viewChild, signal, computed, effect, HostListener, ChangeDetectionStrategy, Component, NgModule } from '@angular/core';
import * as i1 from 'tableau-ui-angular/button';
import { TableauUiButtonModule } from 'tableau-ui-angular/button';
import * as i2 from 'tableau-ui-angular/common';
import { TableauUiCommonModule } from 'tableau-ui-angular/common';
import { TableauUiIconModule } from 'tableau-ui-angular/icon';

class ArrowScrollComponent {
    /**
     * The type of scroll behavior to use.
     * 'default' uses the default scroll behavior,
     * 'arrow' uses arrow buttons for scrolling.
     */
    $scrollType = input.required({
        alias: 'scrollType',
    });
    /**
     * The direction of the scroll.
     * 'horizontal' scrolls horizontally,
     * 'vertical' scrolls vertically.
     */
    $scrollDirection = input.required({
        alias: 'scrollDirection',
    });
    /**
     * Gap between the scroll buttons the scroll container, can be adjusted to fit design requirements.
     * @default '0.25rem'
     */
    $buttonsGap = input('0.25rem', {
        alias: 'buttonsGap',
    });
    /**
     * Color of the scroll buttons, can be adjusted to fit design requirements.
     * @default 'plain'
     */
    $buttonsColor = input('plain', {
        alias: 'buttonsColor',
    });
    /**
     * CSS color for the scroll button icons, can be adjusted to fit design requirements.
     * @default undefined
     */
    $buttonsCssColor = input(undefined, {
        alias: 'buttonsCssColor',
    });
    /**
     * The amount to scroll when the scroll buttons are clicked.
     * This value is in pixels for vertical scrolling and in pixels for horizontal scrolling.
     * @default 100
     */
    $buttonScrollAmount = input(100, {
        alias: 'buttonScrollAmount',
    });
    /**
     * Whether to show a separator between the scroll buttons and the scroll container.
     * @default false
     */
    $showScrollButtonSeparator = input(false, {
        alias: 'showScrollButtonSeparator',
    });
    $scrollContainer = viewChild.required('scrollContainer');
    $scrollContainerDimensions = signal({
        scrollHeight: 0,
        clientHeight: 0,
        scrollWidth: 0,
        clientWidth: 0,
    });
    $scrollArrowsVisible = computed(() => {
        if (this.$scrollType() === 'default') {
            return false;
        }
        const dimensions = this.$scrollContainerDimensions();
        if (this.$scrollDirection() === 'horizontal') {
            return dimensions.scrollWidth > dimensions.clientWidth;
        }
        else {
            return dimensions.scrollHeight > dimensions.clientHeight;
        }
    });
    scrollDirectionChanged = effect(() => {
        const el = this.$scrollContainer().nativeElement;
        if (this.$scrollDirection() === 'horizontal') {
            this.overrideMouseWheelForHorizontal(el);
        }
        else {
            this.removeMouseWheelOverride(el);
        }
    });
    $scrollLeft = signal(0);
    $scrollTop = signal(0);
    $canScroll = computed(() => {
        const el = this.$scrollContainer().nativeElement;
        const dir = this.$scrollDirection();
        const scrollLeft = this.$scrollLeft();
        const scrollTop = this.$scrollTop();
        if (dir === 'horizontal') {
            return {
                back: scrollLeft > 0,
                forward: scrollLeft + el.clientWidth < el.scrollWidth,
            };
        }
        else {
            return {
                back: scrollTop > 0,
                forward: scrollTop + el.clientHeight < el.scrollHeight,
            };
        }
    });
    observer = null;
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    scrollHandler = () => { };
    ngAfterViewInit() {
        const el = this.$scrollContainer().nativeElement;
        this.recalculateScrollContainerDimensions(el);
        this.observer = new ResizeObserver(() => {
            this.recalculateScrollContainerDimensions(el);
        });
        this.observer.observe(this.$scrollContainer().nativeElement);
        this.scrollHandler = () => {
            this.$scrollLeft.set(el.scrollLeft);
            this.$scrollTop.set(el.scrollTop);
        };
        el.addEventListener('scroll', this.scrollHandler, {
            passive: false,
        });
    }
    ngOnDestroy() {
        if (this.observer) {
            this.observer.disconnect();
            this.observer = null;
        }
        const el = this.$scrollContainer().nativeElement;
        this.removeMouseWheelOverride(el);
        el.removeEventListener('scroll', this.scrollHandler);
    }
    onResize() {
        this.recalculateScrollContainerDimensions(this.$scrollContainer().nativeElement);
    }
    recalculateScrollContainerDimensions(el) {
        this.$scrollContainerDimensions.set({
            scrollHeight: el.scrollHeight,
            clientHeight: el.offsetHeight,
            scrollWidth: el.scrollWidth,
            clientWidth: el.clientWidth,
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    wheelHandler = () => { };
    overrideMouseWheelForHorizontal(el) {
        this.wheelHandler = (event) => {
            if (event.deltaY !== 0) {
                event.preventDefault();
                el.scrollBy({
                    left: event.deltaY,
                    behavior: 'smooth',
                });
            }
        };
        el.addEventListener('wheel', this.wheelHandler, {
            passive: false,
        });
    }
    removeMouseWheelOverride(el) {
        el.removeEventListener('wheel', this.wheelHandler);
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        this.wheelHandler = () => { };
    }
    scroll(direction) {
        const el = this.$scrollContainer();
        const amount = this.$buttonScrollAmount();
        if (this.$scrollDirection() === 'horizontal') {
            el.nativeElement.scrollBy({
                left: direction === 'forward' ? amount : amount * -1,
                behavior: 'smooth',
            });
        }
        else {
            el.nativeElement.scrollBy({
                top: direction === 'forward' ? amount : amount * -1,
                behavior: 'smooth',
            });
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: ArrowScrollComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: ArrowScrollComponent, isStandalone: false, selector: "tab-arrow-scroll", inputs: { $scrollType: { classPropertyName: "$scrollType", publicName: "scrollType", isSignal: true, isRequired: true, transformFunction: null }, $scrollDirection: { classPropertyName: "$scrollDirection", publicName: "scrollDirection", isSignal: true, isRequired: true, transformFunction: null }, $buttonsGap: { classPropertyName: "$buttonsGap", publicName: "buttonsGap", isSignal: true, isRequired: false, transformFunction: null }, $buttonsColor: { classPropertyName: "$buttonsColor", publicName: "buttonsColor", isSignal: true, isRequired: false, transformFunction: null }, $buttonsCssColor: { classPropertyName: "$buttonsCssColor", publicName: "buttonsCssColor", isSignal: true, isRequired: false, transformFunction: null }, $buttonScrollAmount: { classPropertyName: "$buttonScrollAmount", publicName: "buttonScrollAmount", isSignal: true, isRequired: false, transformFunction: null }, $showScrollButtonSeparator: { classPropertyName: "$showScrollButtonSeparator", publicName: "showScrollButtonSeparator", isSignal: true, isRequired: false, transformFunction: null } }, host: { listeners: { "window:resize": "onResize()" }, properties: { "style.overflow": "\"hidden\"", "class.horizontal": "$scrollDirection() === \"horizontal\"", "class.vertical": "$scrollDirection() === \"vertical\"" } }, viewQueries: [{ propertyName: "$scrollContainer", first: true, predicate: ["scrollContainer"], descendants: true, isSignal: true }], ngImport: i0, template: "@if ($scrollArrowsVisible()) {\n    <div class=\"scroll-arrow\" [style.margin-bottom]=\"$scrollDirection() === 'vertical' ? $buttonsGap() : '0'\" [style.margin-right]=\"$scrollDirection() === 'horizontal' ? $buttonsGap() : '0'\">\n        <button class=\"back\" layout=\"small-icon\" [color]=\"$buttonsColor()\" [disabled]=\"!$canScroll().back\" [style.color]=\"$buttonsCssColor()\" (click)=\"scroll('back')\">\n            <ng-content select=\"tab-icon[back-button]\" />\n        </button>\n        @if ($showScrollButtonSeparator()) {\n            <tab-separator [direction]=\"$scrollDirection() === 'vertical' ? 'horizontal' : 'vertical'\" />\n        }\n    </div>\n}\n\n<div #scrollContainer class=\"scroll-container\" [class.arrow-scroll]=\"$scrollType() === 'arrow'\" [class.vertical]=\"$scrollDirection() === 'vertical'\" [class.horizontal]=\"$scrollDirection() === 'horizontal'\">\n    <ng-content />\n</div>\n\n@if ($scrollArrowsVisible()) {\n    <div class=\"scroll-arrow\" [style.margin-top]=\"$scrollDirection() === 'vertical' ? $buttonsGap() : '0'\" [style.margin-left]=\"$scrollDirection() === 'horizontal' ? $buttonsGap() : '0'\">\n        @if ($showScrollButtonSeparator()) {\n            <tab-separator [direction]=\"$scrollDirection() === 'vertical' ? 'horizontal' : 'vertical'\" />\n        }\n        <button class=\"forward\" layout=\"small-icon\" [color]=\"$buttonsColor()\" [disabled]=\"!$canScroll().forward\" [style.color]=\"$buttonsCssColor()\" (click)=\"scroll('forward')\">\n            <ng-content select=\"tab-icon[forward-button]\" />\n        </button>\n    </div>\n}\n", styles: [":host{display:grid}:host .scroll-container{max-height:100%;max-width:100%}:host .scroll-container.arrow-scroll{scrollbar-width:none;-ms-overflow-style:none}:host .scroll-container.arrow-scroll::-webkit-scrollbar{display:none}:host.vertical{grid-template-rows:auto 1fr auto}:host.vertical .scroll-arrow{display:flex;flex-direction:column}:host.vertical .scroll-arrow button{width:100%}:host.vertical .scroll-container{grid-row:2;overflow:hidden auto}:host.horizontal{grid-template-columns:auto 1fr auto}:host.horizontal .scroll-arrow{display:flex;flex-direction:row}:host.horizontal .scroll-arrow button{height:100%}:host.horizontal .scroll-container{grid-column:2;overflow:auto hidden}\n"], dependencies: [{ kind: "component", type: i1.ButtonComponent, selector: "button:not([tab-menu-button]),a[button]:not([tab-menu-button])", inputs: ["disabled", "loading", "tabindex", "type", "kind", "color", "layout", "inline", "noFocusOnClick"] }, { kind: "component", type: i2.SeparatorComponent, selector: "tab-separator", inputs: ["direction"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: ArrowScrollComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-arrow-scroll', standalone: false, changeDetection: ChangeDetectionStrategy.OnPush, host: {
                        '[style.overflow]': '"hidden"',
                        '[class.horizontal]': '$scrollDirection() === "horizontal"',
                        '[class.vertical]': '$scrollDirection() === "vertical"',
                    }, template: "@if ($scrollArrowsVisible()) {\n    <div class=\"scroll-arrow\" [style.margin-bottom]=\"$scrollDirection() === 'vertical' ? $buttonsGap() : '0'\" [style.margin-right]=\"$scrollDirection() === 'horizontal' ? $buttonsGap() : '0'\">\n        <button class=\"back\" layout=\"small-icon\" [color]=\"$buttonsColor()\" [disabled]=\"!$canScroll().back\" [style.color]=\"$buttonsCssColor()\" (click)=\"scroll('back')\">\n            <ng-content select=\"tab-icon[back-button]\" />\n        </button>\n        @if ($showScrollButtonSeparator()) {\n            <tab-separator [direction]=\"$scrollDirection() === 'vertical' ? 'horizontal' : 'vertical'\" />\n        }\n    </div>\n}\n\n<div #scrollContainer class=\"scroll-container\" [class.arrow-scroll]=\"$scrollType() === 'arrow'\" [class.vertical]=\"$scrollDirection() === 'vertical'\" [class.horizontal]=\"$scrollDirection() === 'horizontal'\">\n    <ng-content />\n</div>\n\n@if ($scrollArrowsVisible()) {\n    <div class=\"scroll-arrow\" [style.margin-top]=\"$scrollDirection() === 'vertical' ? $buttonsGap() : '0'\" [style.margin-left]=\"$scrollDirection() === 'horizontal' ? $buttonsGap() : '0'\">\n        @if ($showScrollButtonSeparator()) {\n            <tab-separator [direction]=\"$scrollDirection() === 'vertical' ? 'horizontal' : 'vertical'\" />\n        }\n        <button class=\"forward\" layout=\"small-icon\" [color]=\"$buttonsColor()\" [disabled]=\"!$canScroll().forward\" [style.color]=\"$buttonsCssColor()\" (click)=\"scroll('forward')\">\n            <ng-content select=\"tab-icon[forward-button]\" />\n        </button>\n    </div>\n}\n", styles: [":host{display:grid}:host .scroll-container{max-height:100%;max-width:100%}:host .scroll-container.arrow-scroll{scrollbar-width:none;-ms-overflow-style:none}:host .scroll-container.arrow-scroll::-webkit-scrollbar{display:none}:host.vertical{grid-template-rows:auto 1fr auto}:host.vertical .scroll-arrow{display:flex;flex-direction:column}:host.vertical .scroll-arrow button{width:100%}:host.vertical .scroll-container{grid-row:2;overflow:hidden auto}:host.horizontal{grid-template-columns:auto 1fr auto}:host.horizontal .scroll-arrow{display:flex;flex-direction:row}:host.horizontal .scroll-arrow button{height:100%}:host.horizontal .scroll-container{grid-column:2;overflow:auto hidden}\n"] }]
        }], propDecorators: { onResize: [{
                type: HostListener,
                args: ['window:resize']
            }] } });

class TableauUiArrowScrollModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiArrowScrollModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiArrowScrollModule, declarations: [ArrowScrollComponent], imports: [TableauUiButtonModule, TableauUiCommonModule, TableauUiIconModule], exports: [ArrowScrollComponent] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiArrowScrollModule, imports: [TableauUiButtonModule, TableauUiCommonModule, TableauUiIconModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiArrowScrollModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [TableauUiButtonModule, TableauUiCommonModule, TableauUiIconModule],
                    declarations: [ArrowScrollComponent],
                    providers: [],
                    exports: [ArrowScrollComponent],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { ArrowScrollComponent, TableauUiArrowScrollModule };
//# sourceMappingURL=tableau-ui-angular-arrow-scroll.mjs.map
