import * as i0 from '@angular/core';
import { inject, ElementRef, model, contentChildren, contentChild, viewChild, signal, ChangeDetectionStrategy, Component, input, effect, HostListener, Directive, NgModule } from '@angular/core';
import { Subject, fromEvent, map } from 'rxjs';
import { toObservable } from '@angular/core/rxjs-interop';
import { OptionComponent, PrefixComponent, SuffixComponent, TableauUiCommonModule } from 'tableau-ui-angular/common';
import { DialogService, LocalStackOptions, TableauUiDialogModule } from 'tableau-ui-angular/dialog';
import { generateRandomString } from 'tableau-ui-angular/utils';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

class AutoCompleteComponent {
    dialogService = inject(DialogService);
    elementRef = inject(ElementRef);
    dropdownId;
    selectValue$ = new Subject();
    /**
     * The CSS text to apply to the dropdown container
     * @remarks
     * Use this to apply height, maxHeight, etc. to the dropdown container
     * @default '{}'
     */
    $dropdownContainerCss = model({}, {
        alias: 'dropdownContainerCss',
    });
    /**
     * The CSS text to apply to the options container in the dropdown
     * @remarks
     * Use this to apply height, maxHeight, etc. to the dropdown
     * This is applied to the options container, excluding the prefix & suffix
     *
     * @default "{ maxHeight: '300px', height: 'fit-content'}"
     */
    $dropdownOptionsContainerCss = model({
        maxHeight: '300px',
        height: 'fit-content',
    }, {
        alias: 'dropdownOptionsContainerCss',
    });
    /**
     * The template context to use for the dropdown options
     * @remarks
     * Use this to display the 'icon', 'text', and 'hint' properties of the options conditionally
     */
    $dropdownValueTemplateContext = model({
        renderIcon: true,
        renderText: true,
        renderHint: true,
    }, {
        alias: 'dropdownValueTemplateContext',
    });
    $options = contentChildren(OptionComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownPrefix = contentChild(PrefixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownSuffix = contentChild(SuffixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownTemplate = viewChild('dropdownTemplate');
    constructor() {
        const id = generateRandomString();
        this.dropdownId = `dropdown-${id}`;
        toObservable(this.$options).subscribe(() => {
            this.$highlightedOption.set(undefined);
        });
    }
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $openDialog = signal(undefined);
    openDropdown(parentControl) {
        if (parentControl.hasAttribute('disabled') && parentControl.getAttribute('disabled') !== 'false') {
            return undefined;
        }
        if (this.$openDialog()) {
            if (this.$openDialog()?.el === parentControl) {
                return this.$openDialog()?.ref;
            }
            this.$openDialog()?.ref.close();
        }
        const inputRect = parentControl.getBoundingClientRect();
        const ref = this.dialogService.openTemplateDialog(this.$dropdownTemplate(), {
            //top: inputRect.bottom + 'px',
            top(actualWidth, actualHeight) {
                if (inputRect.bottom + actualHeight > window.innerHeight && inputRect.top - actualHeight > 0) {
                    return inputRect.top - actualHeight + 'px';
                }
                return inputRect.bottom + 'px';
            },
            left: inputRect.left + 'px',
            width: inputRect.width + 'px',
            closeOnEscape: true,
            closeOnBackdropClick: true,
            backdropCss: {
                pointerEvents: 'none',
            },
        }, null, new LocalStackOptions(parentControl));
        this.registerKeyNavigation();
        ref.closed$.subscribe(() => {
            this.unregisterKeyNavigation();
            if (this.$openDialog()?.ref === ref) {
                this.$openDialog.set(undefined);
            }
        });
        this.$openDialog.set({ ref, el: parentControl });
        return ref;
    }
    closeDropdown() {
        this.$openDialog()?.ref.close();
    }
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $highlightedOption = signal(undefined);
    optionMouseDown(event) {
        event.preventDefault();
        event.stopPropagation();
    }
    optionKeyNavSubscription;
    registerKeyNavigation() {
        this.unregisterKeyNavigation();
        this.optionKeyNavSubscription = fromEvent(document, 'keydown')
            .pipe(map((e) => e))
            .subscribe((e) => {
            this.onKeyDown(e);
        });
    }
    unregisterKeyNavigation() {
        if (this.optionKeyNavSubscription) {
            this.optionKeyNavSubscription.unsubscribe();
            this.optionKeyNavSubscription = undefined;
        }
    }
    /**
     *
     * @param e Handles KeyDown event for:
     * - host element
     * - global keydown event when dropdown is open
     * @returns
     */
    onKeyDown(e) {
        if (e.key === 'Enter') {
            // if dropdown is not open
            if (this.$openDialog()) {
                // if dropdown is open
                // if an option is highlighted, select it
                if (this.$highlightedOption()) {
                    this.selectValue$.next(this.$highlightedOption());
                }
                else {
                    // if an option is not highlighted, close the dropdown
                    this.$openDialog()?.ref.close();
                }
            }
            e.preventDefault();
            e.stopPropagation();
        }
        if (e.key === 'ArrowUp' || e.key === 'ArrowDown') {
            // if dropdown is open:
            // - we find the currently HIGHLIGHTED option
            // - we find the next option to highlight
            // - if no option is found, we highlight the first/last option
            // if dropdown is NOT open:
            // - if this is a multiselect, ignore
            // - if this is a single select, find the next item to select
            // - if no item is selected, highlight the first/last item
            if (!this.$openDialog()) {
                return;
            }
            let currentIndex = -1;
            if (this.$highlightedOption()) {
                currentIndex = this.$options().findIndex((o) => o.$value() === this.$highlightedOption().$value());
            }
            let nextIndex;
            // find the next index to highlight/select
            if (e.key === 'ArrowDown') {
                if (currentIndex === -1) {
                    // find the first non disabled option
                    nextIndex = this.$options().findIndex((o) => !o.$disabled());
                }
                else {
                    // find the next option that is not disabled
                    nextIndex = this.$options().findIndex((o, i) => i > currentIndex && !o.$disabled());
                    // if no option is found, find the next option that is not disabled before the current item
                    if (nextIndex === -1) {
                        nextIndex = this.$options().findIndex((o, i) => i < currentIndex && !o.$disabled());
                    }
                }
            }
            else if (e.key === 'ArrowUp') {
                if (currentIndex === -1) {
                    // find the last non disabled option
                    nextIndex = this.$options()
                        .slice()
                        .reverse()
                        .findIndex((o) => !o.$disabled());
                    if (nextIndex !== -1) {
                        nextIndex = this.$options().length - nextIndex - 1;
                    }
                }
                else {
                    const flippedCurrentIndex = this.$options().length - currentIndex - 1;
                    // find the next option that is not disabled
                    nextIndex = [...this.$options()].reverse().findIndex((o, i) => i > flippedCurrentIndex && !o.$disabled());
                    // if no option is found, find the next option that is not disabled before the current item
                    if (nextIndex === -1) {
                        nextIndex = [...this.$options()].reverse().findIndex((o, i) => i < flippedCurrentIndex && !o.$disabled());
                    }
                    if (nextIndex !== -1) {
                        nextIndex = this.$options().length - nextIndex - 1;
                    }
                }
            }
            else {
                nextIndex = -1;
            }
            if (nextIndex !== -1) {
                this.$highlightedOption.set(this.$options()[nextIndex]);
                setTimeout(() => document.querySelector(`#${this.dropdownId} .option-wrapper.highlight`)?.scrollIntoView({
                    block: 'nearest',
                }), 10);
            }
            e.preventDefault();
            e.stopPropagation();
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: AutoCompleteComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: AutoCompleteComponent, isStandalone: false, selector: "tab-autocomplete", inputs: { $dropdownContainerCss: { classPropertyName: "$dropdownContainerCss", publicName: "dropdownContainerCss", isSignal: true, isRequired: false, transformFunction: null }, $dropdownOptionsContainerCss: { classPropertyName: "$dropdownOptionsContainerCss", publicName: "dropdownOptionsContainerCss", isSignal: true, isRequired: false, transformFunction: null }, $dropdownValueTemplateContext: { classPropertyName: "$dropdownValueTemplateContext", publicName: "dropdownValueTemplateContext", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { $dropdownContainerCss: "dropdownContainerCssChange", $dropdownOptionsContainerCss: "dropdownOptionsContainerCssChange", $dropdownValueTemplateContext: "dropdownValueTemplateContextChange" }, queries: [{ propertyName: "$options", predicate: OptionComponent, isSignal: true }, { propertyName: "$dropdownPrefix", first: true, predicate: PrefixComponent, descendants: true, isSignal: true }, { propertyName: "$dropdownSuffix", first: true, predicate: SuffixComponent, descendants: true, isSignal: true }], viewQueries: [{ propertyName: "$dropdownTemplate", first: true, predicate: ["dropdownTemplate"], descendants: true, isSignal: true }], ngImport: i0, template: "<ng-template #dropdownTemplate>\n    <div class=\"dropdown-container\" [id]=\"dropdownId\" [ngStyle]=\"$dropdownContainerCss()\">\n        @if ($dropdownPrefix()) {\n            <div class=\"options-prefix\">\n                <ng-content select=\"tab-prefix\" />\n            </div>\n        }\n        @if ($options().length > 0) {\n            <div class=\"option-values\" [ngStyle]=\"$dropdownOptionsContainerCss()\">\n                @for (option of $options(); track option.$value()) {\n                    <!-- eslint-disable-next-line  @angular-eslint/template/interactive-supports-focus, @angular-eslint/template/click-events-have-key-events -->\n                    <div\n                        class=\"option-wrapper\"\n                        [class.disabled]=\"option.$disabled()\"\n                        [class.highlight]=\"!!$highlightedOption() && $highlightedOption()!.$value() === option.$value()\"\n                        (click)=\"selectValue$.next(option)\"\n                        (mousedown)=\"optionMouseDown($event)\"\n                        (mouseenter)=\"$highlightedOption.set(option)\"\n                    >\n                        <div class=\"option-content\">\n                            <ng-container [ngTemplateOutlet]=\"option.$template()\" [ngTemplateOutletContext]=\"$dropdownValueTemplateContext()\" />\n                        </div>\n                    </div>\n                }\n            </div>\n        }\n\n        @if ($dropdownSuffix()) {\n            <div class=\"options-suffix\">\n                <ng-content select=\"tab-suffix\" />\n            </div>\n        }\n    </div>\n</ng-template>\n", styles: [".dropdown-container{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow:hidden auto;-webkit-user-select:none;user-select:none}.dropdown-container.open{display:block}.dropdown-container .option-values{overflow:hidden auto;padding:.5rem}.dropdown-container .option-values .option-wrapper{display:grid;grid-template-columns:auto 1fr auto;align-items:center;border-radius:2px;padding-left:.5em;padding-right:.5em;border:1px solid transparent;margin-top:-1px}.dropdown-container .option-values .option-wrapper .check{font-size:1.2em;width:1.2em;line-height:1;grid-row:1}.dropdown-container .option-values .option-wrapper .check.left{grid-column:1;margin-right:.25em}.dropdown-container .option-values .option-wrapper .check.right{grid-column:3;margin-left:.25em}.dropdown-container .option-values .option-wrapper .check.none{display:none}.dropdown-container .option-values .option-wrapper .option-content{grid-row:1;grid-column:2;padding:.25em 0}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight{background-color:var(--twc-color-highlight);cursor:pointer}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight:not(.selected){outline:var(--twc-menu-highlight-outline);outline-offset:var(--twc-menu-highlight-outline-offset)}.dropdown-container .option-values .option-wrapper.selected{background-color:var(--twc-menu-selected-background-color);outline:var(--twc-menu-selected-outline);outline-offset:var(--twc-menu-selected-outline-offset)}.dropdown-container .option-values .option-wrapper ::ng-deep .tab-option{line-height:normal}\n"], dependencies: [{ kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: AutoCompleteComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-autocomplete', standalone: false, changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-template #dropdownTemplate>\n    <div class=\"dropdown-container\" [id]=\"dropdownId\" [ngStyle]=\"$dropdownContainerCss()\">\n        @if ($dropdownPrefix()) {\n            <div class=\"options-prefix\">\n                <ng-content select=\"tab-prefix\" />\n            </div>\n        }\n        @if ($options().length > 0) {\n            <div class=\"option-values\" [ngStyle]=\"$dropdownOptionsContainerCss()\">\n                @for (option of $options(); track option.$value()) {\n                    <!-- eslint-disable-next-line  @angular-eslint/template/interactive-supports-focus, @angular-eslint/template/click-events-have-key-events -->\n                    <div\n                        class=\"option-wrapper\"\n                        [class.disabled]=\"option.$disabled()\"\n                        [class.highlight]=\"!!$highlightedOption() && $highlightedOption()!.$value() === option.$value()\"\n                        (click)=\"selectValue$.next(option)\"\n                        (mousedown)=\"optionMouseDown($event)\"\n                        (mouseenter)=\"$highlightedOption.set(option)\"\n                    >\n                        <div class=\"option-content\">\n                            <ng-container [ngTemplateOutlet]=\"option.$template()\" [ngTemplateOutletContext]=\"$dropdownValueTemplateContext()\" />\n                        </div>\n                    </div>\n                }\n            </div>\n        }\n\n        @if ($dropdownSuffix()) {\n            <div class=\"options-suffix\">\n                <ng-content select=\"tab-suffix\" />\n            </div>\n        }\n    </div>\n</ng-template>\n", styles: [".dropdown-container{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow:hidden auto;-webkit-user-select:none;user-select:none}.dropdown-container.open{display:block}.dropdown-container .option-values{overflow:hidden auto;padding:.5rem}.dropdown-container .option-values .option-wrapper{display:grid;grid-template-columns:auto 1fr auto;align-items:center;border-radius:2px;padding-left:.5em;padding-right:.5em;border:1px solid transparent;margin-top:-1px}.dropdown-container .option-values .option-wrapper .check{font-size:1.2em;width:1.2em;line-height:1;grid-row:1}.dropdown-container .option-values .option-wrapper .check.left{grid-column:1;margin-right:.25em}.dropdown-container .option-values .option-wrapper .check.right{grid-column:3;margin-left:.25em}.dropdown-container .option-values .option-wrapper .check.none{display:none}.dropdown-container .option-values .option-wrapper .option-content{grid-row:1;grid-column:2;padding:.25em 0}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight{background-color:var(--twc-color-highlight);cursor:pointer}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight:not(.selected){outline:var(--twc-menu-highlight-outline);outline-offset:var(--twc-menu-highlight-outline-offset)}.dropdown-container .option-values .option-wrapper.selected{background-color:var(--twc-menu-selected-background-color);outline:var(--twc-menu-selected-outline);outline-offset:var(--twc-menu-selected-outline-offset)}.dropdown-container .option-values .option-wrapper ::ng-deep .tab-option{line-height:normal}\n"] }]
        }], ctorParameters: () => [] });

class AutoCompleteDirective {
    ref = inject(ElementRef);
    $tabAutoComplete = input.required({
        alias: 'tabAutoComplete',
    });
    autoCompleteSelectedSub = undefined;
    autocompleteChanged = effect(() => {
        const autoComplete = this.$tabAutoComplete();
        if (this.autoCompleteSelectedSub) {
            this.autoCompleteSelectedSub.unsubscribe();
        }
        this.autoCompleteSelectedSub = autoComplete.selectValue$.subscribe((option) => {
            const el = this.ref.nativeElement;
            el.value = option.$value()?.toString() ?? '';
            el.dispatchEvent(new Event('input', { bubbles: true }));
            autoComplete.closeDropdown();
        });
    });
    onFocusIn() {
        this.$tabAutoComplete().openDropdown(this.ref.nativeElement);
    }
    onFocusOut() {
        this.$tabAutoComplete().closeDropdown();
    }
    onInput() {
        this.$tabAutoComplete().openDropdown(this.ref.nativeElement);
    }
    onClick() {
        this.$tabAutoComplete().openDropdown(this.ref.nativeElement);
    }
    ngOnDestroy() {
        this.$tabAutoComplete().closeDropdown();
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: AutoCompleteDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "20.0.3", type: AutoCompleteDirective, isStandalone: false, selector: "input[tabAutoComplete]", inputs: { $tabAutoComplete: { classPropertyName: "$tabAutoComplete", publicName: "tabAutoComplete", isSignal: true, isRequired: true, transformFunction: null } }, host: { listeners: { "focusin": "onFocusIn()", "focusout": "onFocusOut()", "input": "onInput()", "click": "onClick()" } }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: AutoCompleteDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[tabAutoComplete]',
                    standalone: false,
                }]
        }], propDecorators: { onFocusIn: [{
                type: HostListener,
                args: ['focusin']
            }], onFocusOut: [{
                type: HostListener,
                args: ['focusout']
            }], onInput: [{
                type: HostListener,
                args: ['input']
            }], onClick: [{
                type: HostListener,
                args: ['click']
            }] } });

class TableauUiAutoCompleteModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiAutoCompleteModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiAutoCompleteModule, declarations: [AutoCompleteComponent, AutoCompleteDirective], imports: [TableauUiCommonModule, CommonModule, TableauUiDialogModule], exports: [AutoCompleteComponent, AutoCompleteDirective] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiAutoCompleteModule, imports: [TableauUiCommonModule, CommonModule, TableauUiDialogModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiAutoCompleteModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [TableauUiCommonModule, CommonModule, TableauUiDialogModule],
                    declarations: [AutoCompleteComponent, AutoCompleteDirective],
                    exports: [AutoCompleteComponent, AutoCompleteDirective],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { AutoCompleteComponent, AutoCompleteDirective, TableauUiAutoCompleteModule };
//# sourceMappingURL=tableau-ui-angular-autocomplete.mjs.map
