import * as i0 from '@angular/core';
import { input, contentChild, viewChild, inject, ElementRef, Renderer2, signal, ChangeDetectionStrategy, Component, NgModule } from '@angular/core';
import { HintComponent, ErrorComponent, LabelComponent, PrefixComponent, SuffixComponent, TableauUiCommonModule } from 'tableau-ui-angular/common';
import { generateRandomString } from 'tableau-ui-angular/utils';
import { CommonModule } from '@angular/common';
import { ReactiveFormsModule } from '@angular/forms';

class FormFieldComponent {
    inputContainerSelector = 'input,textarea,tab-single-select,tab-multi-select,tab-list-single-select,tab-list-multi-select';
    id = generateRandomString(16);
    $style = input(undefined, {
        alias: 'style',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $hintElement = contentChild(HintComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $errorElement = contentChild(ErrorComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $labelElement = contentChild(LabelComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $prefixElement = contentChild(PrefixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $suffixElement = contentChild(SuffixComponent);
    $prefixContainer = viewChild.required('prefixContainer');
    $suffixContainer = viewChild.required('suffixContainer');
    $inputContainer = viewChild.required('inputContainer');
    host = inject(ElementRef);
    renderer = inject(Renderer2);
    $inputDisabled = signal(false);
    resizeObserver;
    intersectionObserver;
    inputObserver;
    ngAfterContentInit() {
        this.updatePrefixSuffixWidths();
        const inputElement = this.$inputContainer().nativeElement.querySelector(this.inputContainerSelector);
        if (inputElement) {
            inputElement.id = this.id;
            this.updateInputAttributes(inputElement);
            this.inputObserver = new MutationObserver(() => {
                this.updateInputAttributes(inputElement);
            });
            this.inputObserver.observe(inputElement, {
                attributes: true,
                attributeFilter: ['disabled', 'placeholder', 'required'],
            });
        }
    }
    updateInputAttributes(inputElement) {
        this.$inputDisabled.set(inputElement.getAttribute('disabled') != null);
        const required = inputElement.getAttribute('required') != null && inputElement.getAttribute('required') !== 'false';
        const placeholder = inputElement.getAttribute('placeholder');
        if (placeholder !== null) {
            if (required && !placeholder.endsWith('*')) {
                inputElement.setAttribute('placeholder', `${placeholder}*`);
            }
            if (!required && placeholder.endsWith('*')) {
                inputElement.setAttribute('placeholder', placeholder.slice(0, -1));
            }
        }
    }
    ngAfterViewInit() {
        // Recalculate when view initializes
        setTimeout(() => {
            this.updatePrefixSuffixWidths();
        });
        // whenever the formFieldWrapper is in view, update the prefix/suffix widths
        this.intersectionObserver = new IntersectionObserver((entries) => {
            entries.forEach((entry) => {
                if (entry.isIntersecting) {
                    this.updatePrefixSuffixWidths();
                }
            });
        }, { threshold: 0.1 });
        this.intersectionObserver.observe(this.host.nativeElement);
        // Observe size changes in prefix and suffix elements to adjust padding dynamically
        this.resizeObserver = new ResizeObserver(() => {
            this.updatePrefixSuffixWidths();
        });
        this.resizeObserver.observe(this.host.nativeElement);
        if (this.$prefixElement()) {
            this.resizeObserver.observe(this.$prefixContainer().nativeElement);
        }
        if (this.$suffixElement()) {
            this.resizeObserver.observe(this.$suffixContainer().nativeElement);
        }
    }
    ngOnDestroy() {
        if (this.resizeObserver) {
            this.resizeObserver.disconnect();
        }
        if (this.intersectionObserver) {
            this.intersectionObserver.disconnect();
        }
        if (this.inputObserver) {
            this.inputObserver.disconnect();
        }
    }
    updatePrefixSuffixWidths() {
        const prefixElement = this.$prefixElement();
        if (prefixElement) {
            const prefixWidth = prefixElement.elementRef.nativeElement.offsetWidth;
            this.renderer.setStyle(this.$inputContainer().nativeElement.querySelector(this.inputContainerSelector), 'padding-left', `${prefixWidth + 12}px`);
        }
        const suffixElement = this.$suffixElement();
        if (suffixElement) {
            const suffixWidth = suffixElement.elementRef.nativeElement.offsetWidth;
            this.renderer.setStyle(this.$inputContainer().nativeElement.querySelector(this.inputContainerSelector), 'padding-right', `${suffixWidth + 8}px`);
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormFieldComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: FormFieldComponent, isStandalone: false, selector: "tab-form-field", inputs: { $style: { classPropertyName: "$style", publicName: "style", isSignal: true, isRequired: false, transformFunction: null } }, host: { attributes: { "style.display": "grid" }, properties: { "class.disabled": "$inputDisabled()", "attr.aria-disabled": "$inputDisabled()" } }, queries: [{ propertyName: "$hintElement", first: true, predicate: HintComponent, descendants: true, isSignal: true }, { propertyName: "$errorElement", first: true, predicate: ErrorComponent, descendants: true, isSignal: true }, { propertyName: "$labelElement", first: true, predicate: LabelComponent, descendants: true, isSignal: true }, { propertyName: "$prefixElement", first: true, predicate: PrefixComponent, descendants: true, isSignal: true }, { propertyName: "$suffixElement", first: true, predicate: SuffixComponent, descendants: true, isSignal: true }], viewQueries: [{ propertyName: "$prefixContainer", first: true, predicate: ["prefixContainer"], descendants: true, isSignal: true }, { propertyName: "$suffixContainer", first: true, predicate: ["suffixContainer"], descendants: true, isSignal: true }, { propertyName: "$inputContainer", first: true, predicate: ["inputContainer"], descendants: true, isSignal: true }], ngImport: i0, template: "@if ($labelElement()) {\n    <div class=\"form-label\">\n        <label [for]=\"id\"><ng-content select=\"tab-label\" /></label>\n    </div>\n}\n\n<div class=\"form-input-wrapper\">\n    @if ($prefixElement()) {\n        <div #prefixContainer class=\"prefix\">\n            <ng-content select=\"tab-prefix\" />\n        </div>\n    }\n    <div #inputContainer class=\"input-container\">\n        <ng-content />\n    </div>\n    @if ($suffixElement()) {\n        <div #suffixContainer class=\"suffix\">\n            <ng-content select=\"tab-suffix\" />\n        </div>\n    }\n</div>\n@if ($hintElement() || $errorElement()) {\n    <div class=\"form-extra-info\">\n        @if ($hintElement() && (!$errorElement() || $hintElement()!.$showOnError())) {\n            <ng-content select=\"tab-hint\" />\n        }\n        @if ($errorElement()) {\n            <ng-content select=\"tab-error\" />\n        }\n    </div>\n}\n", styles: [":host{display:grid;grid-template-rows:auto 1fr auto;max-height:100%;align-self:start}:host .form-label{margin-bottom:.1rem;color:var(--twc-color-text);grid-row:1}:host .form-input-wrapper{display:flex;align-items:center;position:relative;height:100%;overflow-y:hidden;grid-row:2}:host .form-input-wrapper .prefix,:host .form-input-wrapper .suffix{position:absolute;top:50%;transform:translateY(-50%);pointer-events:none;line-height:0;font-size:1em}:host .form-input-wrapper .prefix{left:.5rem}:host .form-input-wrapper .suffix{right:.5rem}:host .form-input-wrapper .input-container{height:100%;display:grid;width:100%;grid-template-rows:1fr;align-items:center}:host .form-extra-info{position:relative;margin-top:.1rem;font-size:.9em;grid-row:3;display:grid;grid-template-columns:auto 1fr;max-width:100%;overflow:hidden;column-gap:.75rem}:host .form-extra-info ::ng-deep tab-hint{grid-row:1}:host .form-extra-info ::ng-deep tab-hint[type=prefix]{grid-column:1}:host .form-extra-info ::ng-deep tab-hint[type=suffix]{grid-column:2;justify-self:end}:host .form-extra-info ::ng-deep tab-error{grid-column:1}:host.disabled .prefix,:host.disabled .suffix,:host.disabled ::ng-deep .tab-hint,:host.disabled ::ng-deep .tab-label{color:var(--twc-color-disabled)}:host.disabled .prefix::placeholder,:host.disabled .suffix::placeholder,:host.disabled ::ng-deep .tab-hint::placeholder,:host.disabled ::ng-deep .tab-label::placeholder{color:var(--twc-color-disabled)}:host.disabled .prefix::input-placeholder,:host.disabled .suffix::input-placeholder,:host.disabled ::ng-deep .tab-hint::input-placeholder,:host.disabled ::ng-deep .tab-label::input-placeholder{color:var(--twc-color-disabled)}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormFieldComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-form-field', standalone: false, changeDetection: ChangeDetectionStrategy.OnPush, host: {
                        '[class.disabled]': '$inputDisabled()',
                        '[attr.aria-disabled]': '$inputDisabled()',
                        'style.display': 'grid',
                    }, template: "@if ($labelElement()) {\n    <div class=\"form-label\">\n        <label [for]=\"id\"><ng-content select=\"tab-label\" /></label>\n    </div>\n}\n\n<div class=\"form-input-wrapper\">\n    @if ($prefixElement()) {\n        <div #prefixContainer class=\"prefix\">\n            <ng-content select=\"tab-prefix\" />\n        </div>\n    }\n    <div #inputContainer class=\"input-container\">\n        <ng-content />\n    </div>\n    @if ($suffixElement()) {\n        <div #suffixContainer class=\"suffix\">\n            <ng-content select=\"tab-suffix\" />\n        </div>\n    }\n</div>\n@if ($hintElement() || $errorElement()) {\n    <div class=\"form-extra-info\">\n        @if ($hintElement() && (!$errorElement() || $hintElement()!.$showOnError())) {\n            <ng-content select=\"tab-hint\" />\n        }\n        @if ($errorElement()) {\n            <ng-content select=\"tab-error\" />\n        }\n    </div>\n}\n", styles: [":host{display:grid;grid-template-rows:auto 1fr auto;max-height:100%;align-self:start}:host .form-label{margin-bottom:.1rem;color:var(--twc-color-text);grid-row:1}:host .form-input-wrapper{display:flex;align-items:center;position:relative;height:100%;overflow-y:hidden;grid-row:2}:host .form-input-wrapper .prefix,:host .form-input-wrapper .suffix{position:absolute;top:50%;transform:translateY(-50%);pointer-events:none;line-height:0;font-size:1em}:host .form-input-wrapper .prefix{left:.5rem}:host .form-input-wrapper .suffix{right:.5rem}:host .form-input-wrapper .input-container{height:100%;display:grid;width:100%;grid-template-rows:1fr;align-items:center}:host .form-extra-info{position:relative;margin-top:.1rem;font-size:.9em;grid-row:3;display:grid;grid-template-columns:auto 1fr;max-width:100%;overflow:hidden;column-gap:.75rem}:host .form-extra-info ::ng-deep tab-hint{grid-row:1}:host .form-extra-info ::ng-deep tab-hint[type=prefix]{grid-column:1}:host .form-extra-info ::ng-deep tab-hint[type=suffix]{grid-column:2;justify-self:end}:host .form-extra-info ::ng-deep tab-error{grid-column:1}:host.disabled .prefix,:host.disabled .suffix,:host.disabled ::ng-deep .tab-hint,:host.disabled ::ng-deep .tab-label{color:var(--twc-color-disabled)}:host.disabled .prefix::placeholder,:host.disabled .suffix::placeholder,:host.disabled ::ng-deep .tab-hint::placeholder,:host.disabled ::ng-deep .tab-label::placeholder{color:var(--twc-color-disabled)}:host.disabled .prefix::input-placeholder,:host.disabled .suffix::input-placeholder,:host.disabled ::ng-deep .tab-hint::input-placeholder,:host.disabled ::ng-deep .tab-label::input-placeholder{color:var(--twc-color-disabled)}\n"] }]
        }] });

class TableauUiFormFieldModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormFieldModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormFieldModule, declarations: [FormFieldComponent], imports: [CommonModule, TableauUiCommonModule, ReactiveFormsModule], exports: [FormFieldComponent, ReactiveFormsModule, CommonModule] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormFieldModule, imports: [CommonModule, TableauUiCommonModule, ReactiveFormsModule, ReactiveFormsModule, CommonModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormFieldModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, TableauUiCommonModule, ReactiveFormsModule],
                    declarations: [FormFieldComponent],
                    exports: [FormFieldComponent, ReactiveFormsModule, CommonModule],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { FormFieldComponent, TableauUiFormFieldModule };
//# sourceMappingURL=tableau-ui-angular-form-field.mjs.map
