import * as i0 from '@angular/core';
import { signal, Injectable, Pipe, NgModule } from '@angular/core';
import { BehaviorSubject, map, distinctUntilChanged, firstValueFrom, filter, startWith, pairwise, combineLatest, of } from 'rxjs';
import { FormControl, FormResetEvent, FormGroup, FormArray, ReactiveFormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

/* eslint-disable @typescript-eslint/no-explicit-any */
class HierarchyImpl {
    ctrl;
    constructor(ctrl, initialChildren, subscriptions = []) {
        this.ctrl = ctrl;
        this.childList$ = new BehaviorSubject(initialChildren);
        subscriptions.push(this.childList$.subscribe((c) => {
            this.$childList.set(c);
        }));
    }
    /**
     *  The parent of this control
     */
    parent;
    /**
     * The root of this control. This is the top level control in the tree.
     */
    get root() {
        let { parent } = this;
        while (parent) {
            if (parent.hierarchy.parent) {
                parent = parent.hierarchy.parent;
            }
            else {
                return parent;
            }
        }
        return undefined;
    }
    childList$;
    $childList = signal([]);
    getChild(path) {
        if (path === undefined) {
            return HierarchyImpl._getChild$(this.ctrl, []);
        }
        if (typeof path === 'string') {
            path = path.split('.').filter((p) => p !== '');
        }
        return HierarchyImpl._getChild$(this.ctrl, path);
    }
    static _getChild$(form, parts) {
        const key = parts.shift();
        if (key === undefined) {
            return form;
        }
        if (form.type === 'array') {
            const index = parseInt(key, 10);
            if (isNaN(parseInt(key, 10))) {
                return undefined;
            }
            return this._getChild$(form.$controls()[index], parts);
        }
        if (form.type === 'group') {
            const control = form.controls[key];
            if (control === undefined) {
                return undefined;
            }
            return this._getChild$(control, parts);
        }
        return undefined;
    }
    get hierarchyData() {
        return HierarchyImpl._getHierarchyData(this.ctrl);
    }
    static _getHierarchyData(control, name = '') {
        const ret = {
            name: name,
            status: control.$meta().validity,
            value: undefined,
            meta: control.$meta(),
        };
        if (control.type === 'control') {
            const c = control;
            ret.value = c.$value();
        }
        if (control.type === 'group') {
            const group = control;
            ret.value = group.$value();
            ret.children = Object.keys(group.controls).reduce((acc, key) => {
                const child = group.controls[key];
                if (child !== undefined) {
                    acc[key] = this._getHierarchyData(child, key);
                }
                return acc;
            }, {});
        }
        if (control.type === 'array') {
            const arr = control;
            ret.children = arr.$controls().reduce((acc, child, index) => {
                if (child !== undefined) {
                    acc[index] = this._getHierarchyData(child, index.toString());
                }
                return acc;
            }, {});
        }
        return ret;
    }
}

class MetaImpl {
    untouched;
    touched;
    validity;
    pristine;
    dirty;
    enabled;
    disabled;
    updateOn;
    errors;
    childControls;
    constructor(untouched, touched, validity, pristine, dirty, enabled, disabled, updateOn, errors, childControls) {
        this.untouched = untouched;
        this.touched = touched;
        this.validity = validity;
        this.pristine = pristine;
        this.dirty = dirty;
        this.enabled = enabled;
        this.disabled = disabled;
        this.updateOn = updateOn;
        this.errors = errors;
        this.childControls = childControls;
    }
    static fromControl(control) {
        const c = control.control;
        return new MetaImpl(c.untouched, c.touched, c.status, c.pristine, c.dirty, c.enabled, c.disabled, c.updateOn, c.errors, control.hierarchy.childList$.value.map((ca) => MetaImpl.fromControl(ca)));
    }
    hasError(errorCode, requireTouched = true) {
        return MetaImpl._hasError(this, errorCode, requireTouched);
    }
    getErrorValue(errorCode, requireTouched = true) {
        return MetaImpl._getErrorValue(this, errorCode, requireTouched);
    }
    static _hasError(meta, errorCode, requireTouched = true) {
        if (!meta) {
            return false;
        }
        if ((!requireTouched || meta.touched) && meta.validity === 'INVALID' && meta.errors && (errorCode === undefined ? Object.keys(meta.errors).length > 0 : errorCode in meta.errors)) {
            return true;
        }
        if (meta.childControls) {
            return meta.childControls.some((c) => this._hasError(c, errorCode, requireTouched));
        }
        return false;
    }
    static _getErrorValue(meta, errorCode, requireTouched = true) {
        if (meta === undefined) {
            return null;
        }
        if ((!requireTouched || meta.touched) && meta.validity === 'INVALID' && meta.errors && errorCode in meta.errors) {
            return meta.errors[errorCode];
        }
        if (meta.childControls) {
            for (const c of meta.childControls) {
                const value = this._getErrorValue(c, errorCode, requireTouched);
                if (value !== undefined) {
                    return value;
                }
            }
        }
        return null;
    }
    static compare(a, b) {
        const baseEqual = a.touched === b.touched && a.validity === b.validity && a.dirty === b.dirty && a.enabled === b.enabled && this.validationErrorsEqual(a.errors, b.errors);
        if (!baseEqual) {
            return false;
        }
        if (!a.childControls && !b.childControls) {
            return true;
        }
        if (!a.childControls || !b.childControls) {
            return false;
        }
        if (a.childControls.length !== b.childControls.length) {
            return false;
        }
        for (let i = 0; i < a.childControls.length; i++) {
            if (!this.compare(a.childControls[i], b.childControls[i])) {
                return false;
            }
        }
        return true;
    }
    static validationErrorsEqual(a, b) {
        if (a === b) {
            return true;
        }
        if (!a || !b) {
            return false;
        }
        const aKeys = Object.keys(a);
        const bKeys = Object.keys(b);
        if (aKeys.length !== bKeys.length) {
            return false;
        }
        for (const key of aKeys) {
            if (!(key in b)) {
                return false;
            }
            if (typeof a[key] !== typeof b[key]) {
                return false;
            }
            if (a[key] !== b[key]) {
                return false;
            }
        }
        return true;
    }
}
class MetaFnsImpl {
    control;
    ctrl;
    constructor(control, ctrl) {
        this.control = control;
        this.ctrl = ctrl;
    }
    /**
     * Sets errors on a form control when running validations manually, rather than automatically.
     *
     * Calling `setErrors` also updates the validity of the parent control.
     *
     * @param opts Configuration options that determine how the control propagates
     * changes and emits events after the control errors are set.
     * * `emitEvent`: When true or not supplied (the default), the `statusChanges`
     * observable emits an event after the errors are set.
     *
     * @usageNotes
     *
     * ### Manually set the errors for a control
     *
     * ```ts
     * const login = new FormControl('someLogin');
     * login.setErrors({
     *   notUnique: true
     * });
     *
     * expect(login.valid).toEqual(false);
     * expect(login.errors).toEqual({ notUnique: true });
     *
     * login.setValue('someOtherLogin');
     *
     * expect(login.valid).toEqual(true);
     * ```
     */
    setErrors(errors, emitEvent = true) {
        this.control.setErrors(errors, { emitEvent });
        return this.ctrl;
    }
    /**
     * Marks the control as `touched`. A control is touched by focus and
     * blur events that do not change the value.
     *
     *
     * @see {@link markAsUntouched()}
     * @see {@link markAsDirty()}
     * @see {@link markAsPristine()}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     * @param markAncestors When true, marks all ancestors of the control as well
     */
    markAsTouched(markAncestors = false, emitEvent = true) {
        if (!markAncestors && this.control.touched) {
            return this.ctrl;
        }
        this.control.markAsTouched({
            onlySelf: !markAncestors,
            emitEvent: emitEvent,
        });
        return this.ctrl;
    }
    /**
     * Marks the control and all its descendant controls as `touched`.
     * @see {@link markAsTouched()}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     */
    markAllAsTouched(emitEvent = true) {
        this.control.markAllAsTouched({
            emitEvent: emitEvent,
        });
        return this.ctrl;
    }
    /**
     * Marks the control as `untouched`.
     *
     * If the control has any children, also marks all children as `untouched`
     * and recalculates the `touched` status of all parent controls.
     *
     * @see {@link markAsTouched()}
     * @see {@link markAsDirty()}
     * @see {@link markAsPristine()}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     * @param markAncestors When true, mark only this control. When false or not supplied,
     * marks all direct ancestors. Default is false.
     */
    markAsUntouched(emitEvent = true, markAncestors = false) {
        this.control.markAsUntouched({
            onlySelf: !markAncestors,
            emitEvent: emitEvent,
        });
        return this.ctrl;
    }
    /**
     * Marks the control as `dirty`. A control becomes dirty when
     * the control's value is changed through the UI; compare `markAsTouched`.
     *
     * If the control is already dirty this does nothing
     *
     * @see {@link markAsTouched()}
     * @see {@link markAsUntouched()}
     * @see {@link markAsPristine()}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     * @param markAncestors When true, mark only this control. When false or not supplied,
     * marks all direct ancestors. Default is false.
     */
    markAsDirty(emitEvent = true, markAncestors = false) {
        if (this.ctrl.meta$.value.dirty) {
            return this.ctrl;
        }
        this.control.markAsDirty({
            onlySelf: !markAncestors,
            emitEvent: emitEvent,
        });
        return this.ctrl;
    }
    /**
     * Marks the control as `pristine`.
     *
     * If the control has any children, marks all children as `pristine`,
     * and recalculates the `pristine` status of all parent
     * controls.
     *
     * If the control is already pristine, this does nothing
     *
     * @see {@link markAsTouched()}
     * @see {@link markAsUntouched()}
     * @see {@link markAsDirty()}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     * @param markAncestors When true, mark only this control. When false or not supplied,
     * marks all direct ancestors. Default is false.
     */
    markAsPristine(emitEvent = true, markAncestors = false) {
        if (this.ctrl.meta$.value.pristine) {
            return this.ctrl;
        }
        this.control.markAsPristine({
            onlySelf: !markAncestors,
            emitEvent: emitEvent,
        });
        return this.ctrl;
    }
    /**
     * Disables the control. This means the control is exempt from validation checks and
     * excluded from the aggregate value of any parent. Its validity is `DISABLED`.
     *
     * If the control has children, all children are also disabled.
     *
     * If the control is already disabled, this does nothing.
     *
     * @see {@link AbstractControlMeta.validity}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     * @param markAncestors When true, mark only this control. When false or not supplied,
     * marks all direct ancestors. Default is false.
     */
    disable(emitEvent = true, markAncestors = false) {
        if (!this.control.enabled) {
            return this.ctrl;
        }
        this.control.disable({
            onlySelf: !markAncestors,
            emitEvent: emitEvent,
        });
        this.control.disable({ onlySelf: !markAncestors });
        return this.ctrl;
    }
    /**
     * Enables the control. This means the control is included in validation checks and
     * the aggregate value of its parent. Its status recalculates based on its value and
     * its validators.
     *
     * By default, if the control has children, all children are enabled.
     *
     * If the control is already enabled, this does nothing.
     *
     * @see {@link AbstractControl.status}
     *
     * @param emitEvent When true, the meta$ observable emits an event. Default is true.
     * @param markAncestors When true, mark only this control. When false or not supplied,
     * marks all direct ancestors. Default is false.
     */
    enable(emitEvent = true, markAncestors = false) {
        if (this.control.enabled) {
            return this.ctrl;
        }
        this.control.enable({
            onlySelf: !markAncestors,
            emitEvent: emitEvent,
        });
        return this.ctrl;
    }
}

class ACImpl {
    type;
    control;
    subscriptions = [];
    meta$;
    $meta;
    // readonly rawValue$: ReadonlyBehaviorSubject<unknown>;
    // readonly $rawValue: Signal<unknown>;
    hierarchy;
    constructor(type, control, childList = []) {
        this.control = control;
        this.hierarchy = new HierarchyImpl(this, childList);
        this.subscriptions.push(this.hierarchy.childList$.subscribe((ch) => {
            ch.forEach((c) => {
                c.hierarchy.parent = this;
            });
        }));
        this.type = type;
        const initialMeta = MetaImpl.fromControl(this);
        this.meta$ = new BehaviorSubject(initialMeta);
        this.$meta = signal(initialMeta);
        this.subscriptions.push(this.control.events
            .pipe(map(() => MetaImpl.fromControl(this)), distinctUntilChanged((a, b) => MetaImpl.compare(a, b)))
            .subscribe((meta) => {
            this.meta$.next(meta);
            this.$meta.set(meta);
        }));
    }
    getRawValue() {
        return this.control.getRawValue();
    }
    // public setValue(
    //     value: TValue,
    //     options?: {
    //         onlySelf?: boolean;
    //         emitEvent?: boolean;
    //         emitModelToViewChange?: boolean;
    //         emitViewToModelChange?: boolean;
    //     },
    // ) {
    //     this.control.setValue(value, options);
    // }
    async isInvalid() {
        return firstValueFrom(this.meta$.pipe(map((e) => {
            if (e.validity === 'INVALID') {
                return true;
            }
            else if (e.validity !== 'PENDING') {
                return false;
            }
            return undefined;
        }), filter((e) => e !== undefined)));
    }
    updateValueAndValidity(includeAncestors, includeDescendants, markAsTouched, markAsDirty, emitEvent = true) {
        ACImpl._updateValueAndValidity(this, includeAncestors, includeDescendants, markAsTouched, markAsDirty, emitEvent);
    }
    static _updateValueAndValidity(f, includeAncestors, includeDescendants, markAsTouched, markAsDirty, emitEvent) {
        if (includeDescendants) {
            for (const child of f.hierarchy.childList$.value) {
                this._updateValueAndValidity(child, false, includeDescendants, markAsTouched, markAsDirty, emitEvent);
            }
        }
        if (markAsTouched) {
            f.control.markAsTouched({ onlySelf: true, emitEvent: false });
        }
        if (markAsDirty) {
            f.control.markAsDirty({ onlySelf: true, emitEvent: false });
        }
        f.control.updateValueAndValidity({ onlySelf: true, emitEvent });
        if (includeAncestors) {
            let { parent } = f.hierarchy;
            while (parent) {
                this._updateValueAndValidity(parent, false, false, markAsTouched, markAsDirty, emitEvent);
                parent = parent.hierarchy.parent;
            }
        }
    }
    destroy() {
        this.subscriptions.forEach((sub) => {
            sub.unsubscribe();
        });
        this.subscriptions.length = 0;
        this.hierarchy.childList$.value.forEach((child) => {
            child.destroy();
        });
    }
}

class ValidatorFnsImpl {
    control;
    constructor(control) {
        this.control = control;
    }
    /**
     * Returns the function that is used to determine the validity of this control synchronously.
     * If multiple validators have been added, this will be a single composed function.
     * See `Validators.compose()` for additional information.
     */
    get validator() {
        return this.control.validator;
    }
    /**
     * Sets the function that is used to determine the validity of this control synchronously.
     * It's recommended to use 'setValidators' or 'addValidators' instead, which will compose this function
     */
    set validator(validator) {
        this.control.validator = validator;
    }
    /**
     * Returns the function that is used to determine the validity of this control asynchronously.
     * If multiple validators have been added, this will be a single composed function.
     * See `Validators.compose()` for additional information.
     */
    get asyncValidator() {
        return this.control.asyncValidator;
    }
    /**
     * Sets the function that is used to determine the validity of this control asynchronously.
     * It's recommended to use 'setAsyncValidators' or 'addAsyncValidators' instead, which will compose this function
     */
    set asyncValidator(asyncValidatorFn) {
        this.control.asyncValidator = asyncValidatorFn;
    }
    /**
     * Sets the synchronous validators that are active on this control.  Calling
     * this overwrites any existing synchronous validators.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     * If you want to add a new validator without affecting existing ones, consider
     * using `addValidators()` method instead.
     */
    setValidators(validators) {
        this.control.setValidators(validators);
        return this.control;
    }
    /**
     * Sets the asynchronous validators that are active on this control. Calling this
     * overwrites any existing asynchronous validators.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     * If you want to add a new validator without affecting existing ones, consider
     * using `addAsyncValidators()` method instead.
     */
    setAsyncValidators(validators) {
        this.control.setAsyncValidators(validators);
        return this.control;
    }
    /**
     * Add a synchronous validator or validators to this control, without affecting other validators.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     * Adding a validator that already exists will have no effect. If duplicate validator functions
     * are present in the `validators` array, only the first instance would be added to a form
     * control.
     *
     * @param validators The new validator function or functions to add to this control.
     */
    addValidators(validators) {
        this.control.addValidators(validators);
        return this.control;
    }
    /**
     * Add an asynchronous validator or validators to this control, without affecting other
     * validators.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     * Adding a validator that already exists will have no effect.
     *
     * @param validators The new asynchronous validator function or functions to add to this control.
     */
    addAsyncValidators(validators) {
        this.control.addAsyncValidators(validators);
        return this.control;
    }
    /**
     * Remove a synchronous validator from this control, without affecting other validators.
     * Validators are compared by function reference; you must pass a reference to the exact same
     * validator function as the one that was originally set. If a provided validator is not found,
     * it is ignored.
     *
     * @usageNotes
     *
     * ### Reference to a ValidatorFn
     *
     * ```
     * // Reference to the RequiredValidator
     * const ctrl = new FormControl<string | null>('', Validators.required);
     * ctrl.removeValidators(Validators.required);
     *
     * // Reference to anonymous function inside MinValidator
     * const minValidator = Validators.min(3);
     * const ctrl = new FormControl<string | null>('', minValidator);
     * expect(ctrl.hasValidator(minValidator)).toEqual(true)
     * expect(ctrl.hasValidator(Validators.min(3))).toEqual(false)
     *
     * ctrl.removeValidators(minValidator);
     * ```
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     * @param validators The validator or validators to remove.
     */
    removeValidators(validators) {
        this.control.removeValidators(validators);
        return this.control;
    }
    /**
     * Remove an asynchronous validator from this control, without affecting other validators.
     * Validators are compared by function reference; you must pass a reference to the exact same
     * validator function as the one that was originally set. If a provided validator is not found, it
     * is ignored.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     * @param validators The asynchronous validator or validators to remove.
     */
    removeAsyncValidators(validators) {
        this.control.removeAsyncValidators(validators);
        return this.control;
    }
    /**
     * Check whether a synchronous validator function is present on this control. The provided
     * validator must be a reference to the exact same function that was provided.
     *
     * @usageNotes
     *
     * ### Reference to a ValidatorFn
     *
     * ```
     * // Reference to the RequiredValidator
     * const ctrl = new FormControl<number | null>(0, Validators.required);
     * expect(ctrl.hasValidator(Validators.required)).toEqual(true)
     *
     * // Reference to anonymous function inside MinValidator
     * const minValidator = Validators.min(3);
     * const ctrl = new FormControl<number | null>(0, minValidator);
     * expect(ctrl.hasValidator(minValidator)).toEqual(true)
     * expect(ctrl.hasValidator(Validators.min(3))).toEqual(false)
     * ```
     *
     * @param validator The validator to check for presence. Compared by function reference.
     * @returns Whether the provided validator was found on this control.
     */
    hasValidator(validator) {
        return this.control.hasValidator(validator);
    }
    /**
     * Check whether an asynchronous validator function is present on this control. The provided
     * validator must be a reference to the exact same function that was provided.
     *
     * @param validator The asynchronous validator to check for presence. Compared by function
     *     reference.
     * @returns Whether the provided asynchronous validator was found on this control.
     */
    hasAsyncValidator(validator) {
        return this.control.hasAsyncValidator(validator);
    }
    /**
     * Empties out the synchronous validator list.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     */
    clearValidators() {
        this.control.clearValidators();
        return this.control;
    }
    /**
     * Empties out the async validator list.
     *
     * When you add or remove a validator at run time, you must call
     * `updateValueAndValidity()` for the new validation to take effect.
     *
     */
    clearAsyncValidators() {
        this.control.clearAsyncValidators();
        return this.control;
    }
}

class RegisterFnsImpl {
    fgControl;
    subscriptions;
    constructor(fgControl, subscriptions) {
        this.fgControl = fgControl;
        this.subscriptions = subscriptions;
    }
    enableChange(callback) {
        this.subscriptions.push(this.fgControl.meta$
            .pipe(map((e) => e.enabled), distinctUntilChanged())
            .subscribe((e) => {
            callback(e);
        }));
        return this.fgControl;
    }
    metaChange(callback) {
        this.subscriptions.push(this.fgControl.meta$.subscribe((meta) => {
            callback(meta);
        }));
        return this.fgControl;
    }
    alwaysDisabled() {
        this.fgControl.metaFn.disable();
        this.subscriptions.push(this.fgControl.meta$.subscribe((meta) => {
            if (meta.enabled) {
                this.fgControl.metaFn.disable();
            }
        }));
        return this.fgControl;
    }
}

class FcRegisterFnsImpl extends RegisterFnsImpl {
    fcControl;
    constructor(fcControl, subscriptions = []) {
        super(fcControl, subscriptions);
        this.fcControl = fcControl;
    }
    valueChange(callback, alsoRunOnEnabled = false, alsoRunOnDisabled = false) {
        const subs = [
            this.fcControl.value$.pipe(startWith(undefined), pairwise(), map((v) => [v[0], v[1] ?? v[0]])),
        ];
        if (alsoRunOnEnabled || alsoRunOnDisabled) {
            subs.push(this.fcControl.meta$.pipe(map((e) => e.enabled), filter((enabled) => {
                if (enabled && alsoRunOnEnabled) {
                    return true;
                }
                else if (!enabled && alsoRunOnDisabled) {
                    return true;
                }
                return false;
            })));
        }
        this.subscriptions.push(combineLatest(subs).subscribe(([v]) => {
            callback(v[1], v[0], this.fcControl);
        }));
        return this.fcControl;
    }
}

class FCImpl extends ACImpl {
    _value$;
    $_value;
    get value$() {
        return this._value$.asObservable();
    }
    get $value() {
        return this.$_value;
    }
    submitted$;
    reset$;
    validatorFn;
    metaFn;
    registerFn;
    _control;
    _defaultValue;
    constructor(params) {
        const control = new FormControl({
            value: params.defaultValue,
            disabled: params.initialDisabled ?? false,
        }, {
            nonNullable: true,
            updateOn: params.updateOn,
            asyncValidators: params.asyncValidators,
            validators: params.validators,
        });
        super('control', control, []);
        this._control = control;
        this._defaultValue = params.defaultValue;
        this._value$ = new BehaviorSubject(params.defaultValue);
        this.$_value = signal(this._value$.value);
        this.subscriptions.push(control.valueChanges
            .pipe(startWith(control.value), distinctUntilChanged((a, b) => {
            // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
            if (!a && !b) {
                return true;
            }
            // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
            if (!a || !b) {
                return false;
            }
            if (Array.isArray(a) && Array.isArray(b)) {
                if (a.length !== b.length) {
                    return false;
                }
                for (let i = 0; i < a.length; i++) {
                    if (a[i] !== b[i]) {
                        return false;
                    }
                }
                return true;
            }
            return a === b;
        }))
            .subscribe((v) => {
            this._value$.next(v);
        }));
        this.subscriptions.push(this._value$.subscribe((v) => {
            this.$_value.set(v);
        }));
        this.submitted$ = this.control.events.pipe(filter((e) => e instanceof SubmitEvent), map(() => this._value$.value));
        this.reset$ = this.control.events.pipe(filter((e) => e instanceof FormResetEvent), map(() => this._value$.value));
        this.registerFn = new FcRegisterFnsImpl(this, this.subscriptions);
        this.validatorFn = new ValidatorFnsImpl(this.control);
        this.metaFn = new MetaFnsImpl(this.control, this);
    }
    setValue(value, options) {
        this.control.setValue(value, {
            onlySelf: options?.onlySelf ?? false,
            emitEvent: options?.emitEvent ?? true,
            emitModelToViewChange: options?.emitModelToViewChange ?? true,
            emitViewToModelChange: options?.emitViewToModelChange ?? true,
        });
    }
    resetWithDefaultValue(updateParentsValue = true, emitEvent = true) {
        this._control.reset(this._defaultValue, { onlySelf: !updateParentsValue, emitEvent: emitEvent });
    }
    reset(value, updateParentsValue = true, emitEvent = true) {
        this._control.reset(value, { onlySelf: !updateParentsValue, emitEvent: emitEvent });
    }
}

class FgRegisterFnsImpl extends RegisterFnsImpl {
    fg;
    constructor(fg, subscriptions = []) {
        super(fg, subscriptions);
        this.fg = fg;
    }
    valueChange(callback, alsoRunOnEnabled = false, alsoRunOnDisabled = false) {
        const subs = [
            this.fg.value$.pipe(startWith(undefined), pairwise(), map((v) => [v[0], v[1] ?? v[0]])),
        ];
        if (alsoRunOnEnabled || alsoRunOnDisabled) {
            subs.push(this.fg.meta$.pipe(map((e) => e.enabled), filter((enabled) => {
                if (enabled && alsoRunOnEnabled) {
                    return true;
                }
                else if (!enabled && alsoRunOnDisabled) {
                    return true;
                }
                return false;
            })));
        }
        this.subscriptions.push(combineLatest(subs).subscribe(([v]) => {
            callback(v[1], v[0], this.fg);
        }));
        return this.fg;
    }
    childChange(formControlSelector, callback, alsoRunOnEnabled = false, alsoRunOnDisabled = false) {
        const ctrl = formControlSelector(this.fg.controls);
        const subs = [
            ctrl.value$.pipe(map((v) => {
                return v;
            })),
        ];
        if (alsoRunOnEnabled || alsoRunOnDisabled) {
            subs.push(this.fg.meta$.pipe(map((e) => e.enabled), filter((enabled) => {
                if (enabled && alsoRunOnEnabled) {
                    return true;
                }
                else if (!enabled && alsoRunOnDisabled) {
                    return true;
                }
                return false;
            })));
        }
        this.subscriptions.push(combineLatest(subs).subscribe(([v]) => {
            callback(this.fg, ctrl, v);
        }));
        return this.fg;
    }
    childGroupChange(formControlSelector, callback, alsoRunOnEnabled = false, alsoRunOnDisabled = false) {
        const ctrl = formControlSelector(this.fg.controls);
        const subs = [ctrl.value$];
        if (alsoRunOnEnabled || alsoRunOnDisabled) {
            subs.push(this.fg.meta$.pipe(map((e) => e.enabled), filter((enabled) => {
                if (enabled && alsoRunOnEnabled) {
                    return true;
                }
                else if (!enabled && alsoRunOnDisabled) {
                    return true;
                }
                return false;
            })));
        }
        this.subscriptions.push(combineLatest(subs).subscribe(([v]) => {
            callback(this.fg, ctrl, v);
        }));
        return this.fg;
    }
}

class FGImpl extends ACImpl {
    _value$;
    $_value;
    _rawValue$;
    $_rawValue;
    get value$() {
        return this._value$.asObservable();
    }
    get $value() {
        return this.$_value;
    }
    get rawValue$() {
        return this._rawValue$.asObservable();
    }
    get $rawValue() {
        return this.$_rawValue;
    }
    controls;
    submitted$;
    reset$;
    validatorFn;
    metaFn;
    registerFn;
    _control;
    _defaultValue;
    constructor(params) {
        const controls = Object.entries(params.controls).reduce((acc, [key, child]) => {
            const c = child.control;
            // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-assignment
            acc[key] = c;
            return acc;
        }, {});
        const control = new FormGroup(controls, {
            validators: params.validators,
            asyncValidators: params.asyncValidators,
            updateOn: params.updateOn,
        });
        const childList = Object.entries(params.controls).map(([, child]) => child);
        super('group', control, childList);
        this._control = control;
        this._defaultValue = control.getRawValue();
        this.controls = params.controls;
        this._value$ = new BehaviorSubject(control.value);
        this.$_value = signal(this._value$.value);
        this._rawValue$ = new BehaviorSubject(this._defaultValue);
        this.$_rawValue = signal(this._rawValue$.value);
        this.subscriptions.push(control.valueChanges.pipe(startWith(control.value)).subscribe((v) => {
            this._value$.next(v);
        }));
        this.subscriptions.push(this._value$.subscribe((v) => {
            this.$_value.set(v);
            const rawValue = control.getRawValue();
            this._rawValue$.next(rawValue);
            this.$_rawValue.set(rawValue);
        }));
        this.submitted$ = this.control.events.pipe(filter((e) => e instanceof SubmitEvent), map(() => this._value$.value));
        this.reset$ = this.control.events.pipe(filter((e) => e instanceof FormResetEvent), map(() => this._value$.value));
        this.registerFn = new FgRegisterFnsImpl(this, this.subscriptions);
        this.validatorFn = new ValidatorFnsImpl(this.control);
        this.metaFn = new MetaFnsImpl(this.control, this);
    }
    setValue(value, options) {
        this.control.setValue(value, {
            onlySelf: options?.onlySelf ?? false,
            emitEvent: options?.emitEvent ?? true,
            emitModelToViewChange: options?.emitModelToViewChange ?? true,
            emitViewToModelChange: options?.emitViewToModelChange ?? true,
        });
    }
    resetWithDefaultValue(updateParentsValue = true, emitEvent = true) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-explicit-any
        this._control.reset(this._defaultValue, { onlySelf: !updateParentsValue, emitEvent: emitEvent });
    }
    reset(value, updateParentsValue = true, emitEvent = true) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-explicit-any
        this._control.reset(value, { onlySelf: !updateParentsValue, emitEvent: emitEvent });
    }
}

class FaRegisterFnsImpl extends RegisterFnsImpl {
    fa;
    constructor(fa, subscriptions = []) {
        super(fa, subscriptions);
        this.fa = fa;
    }
    /**
     * Registers a callback to be called when the value of the array changes.
     * The callback is always called initially.
     * @param callback
     * @param alsoRunOnEnabled Whether to also run the callback when the control is enabled.
     * @param alsoRunOnDisabled Whether to also run the callback when the control is disabled.
     */
    valueChange(callback, alsoRunOnEnabled = false, alsoRunOnDisabled = false) {
        const subs = [
            this.fa.value$.pipe(startWith(undefined), pairwise(), map((v) => [v[0], (v[1] ?? v[0])])),
        ];
        if (alsoRunOnEnabled || alsoRunOnDisabled) {
            subs.push(this.fa.meta$.pipe(map((e) => e.enabled), filter((enabled) => {
                if (enabled && alsoRunOnEnabled) {
                    return true;
                }
                else if (!enabled && alsoRunOnDisabled) {
                    return true;
                }
                return false;
            })));
        }
        this.subscriptions.push(combineLatest(subs).subscribe(([v]) => {
            callback(v[1], v[0], this.fa.$controls(), this.fa);
        }));
        return this.fa;
    }
}

class FAImpl extends ACImpl {
    $_value;
    _value$;
    _rawValue$;
    $_rawValue;
    get value$() {
        return this._value$.asObservable();
    }
    get $value() {
        return this.$_value;
    }
    get rawValue$() {
        return this._rawValue$.asObservable();
    }
    get $rawValue() {
        return this.$_rawValue;
    }
    get _controls$() {
        return this.hierarchy.childList$;
    }
    get controls$() {
        return this._controls$;
    }
    $_controls;
    get $controls() {
        return this.$_controls;
    }
    submitted$;
    reset$;
    validatorFn;
    metaFn;
    registerFn;
    _control;
    _defaultValue;
    constructor(params) {
        const controlsArray = params.controls.map((child) => {
            const c = child.control;
            return c;
        });
        const control = new FormArray(controlsArray, {
            validators: params.validators,
            asyncValidators: params.asyncValidators,
            updateOn: params.updateOn,
        });
        super('array', control, params.controls);
        this._control = control;
        this._defaultValue = control.getRawValue();
        this.$_controls = signal(this._controls$.value);
        this._value$ = new BehaviorSubject(control.value);
        this.$_value = signal(control.value);
        this._rawValue$ = new BehaviorSubject(this._defaultValue);
        this.$_rawValue = signal(this._rawValue$.value);
        this.subscriptions.push(control.valueChanges.pipe(startWith(control.value)).subscribe((v) => {
            this._value$.next(v);
        }));
        this.subscriptions.push(this._value$.subscribe((v) => {
            this.$_value.set(v);
        }));
        this.subscriptions.push(this._controls$.subscribe((v) => {
            this.$_controls.set(v);
            const rawValue = control.getRawValue();
            this._rawValue$.next(rawValue);
            this.$_rawValue.set(rawValue);
        }));
        this.submitted$ = this.control.events.pipe(filter((e) => e instanceof SubmitEvent), map(() => this._value$.value));
        this.reset$ = this.control.events.pipe(filter((e) => e instanceof FormResetEvent), map(() => this._value$.value));
        this.registerFn = new FaRegisterFnsImpl(this, this.subscriptions);
        this.validatorFn = new ValidatorFnsImpl(this.control);
        this.metaFn = new MetaFnsImpl(this.control, this);
    }
    get formArray() {
        return this.control;
    }
    push(control, options) {
        const c = control.control;
        this.formArray.push(c, options);
        const childList = this.hierarchy.childList$.value;
        childList.push(control);
        this.hierarchy.childList$.next(childList);
    }
    removeAt(index, destroyControl, options) {
        this.formArray.removeAt(index, options);
        const childList = this.hierarchy.childList$.value;
        const control = childList.splice(index, 1)[0];
        if (control !== undefined && destroyControl) {
            control.destroy();
        }
        this.hierarchy.childList$.next(childList);
    }
    insert(index, control, options) {
        const c = control.control;
        this.formArray.insert(index, c, options);
        const childList = this.hierarchy.childList$.value;
        childList.splice(index, 0, control);
        this.hierarchy.childList$.next(childList);
    }
    clear(destroyControls, options) {
        this.formArray.clear(options);
        const childList = this.hierarchy.childList$.value;
        if (destroyControls) {
            childList.forEach((child) => {
                child.destroy();
            });
        }
        this.hierarchy.childList$.next([]);
    }
    at(index) {
        return this._controls$.value[index];
    }
    resetWithDefaultValue(updateParentsValue = true, emitEvent = true) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-explicit-any
        this._control.reset(this._defaultValue, { onlySelf: !updateParentsValue, emitEvent: emitEvent });
    }
    reset(value, updateParentsValue = true, emitEvent = true) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-explicit-any
        this._control.reset(value, { onlySelf: !updateParentsValue, emitEvent: emitEvent });
    }
}

class FB {
    control(value, validators, asyncValidators, initialDisabled, updateOn) {
        return new FCImpl({
            defaultValue: value,
            validators: validators,
            asyncValidators: asyncValidators,
            initialDisabled: initialDisabled,
            updateOn: updateOn,
        });
    }
    group(controls, validators, asyncValidators, updateOn) {
        return new FGImpl({
            controls: controls,
            validators: validators,
            asyncValidators: asyncValidators,
            updateOn: updateOn,
        });
    }
    array(controls, validators, asyncValidators, updateOn) {
        return new FAImpl({
            controls: controls,
            validators: validators,
            asyncValidators: asyncValidators,
            updateOn: updateOn,
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FB, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FB, providedIn: 'any' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FB, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'any',
                }]
        }] });

class FormErrorPipe {
    transform(meta, specificError, requireTouched = true, log = false) {
        const ret = meta?.getErrorValue(specificError, requireTouched);
        if (log) {
            console.log('getFormError', meta, specificError, requireTouched, ret);
        }
        return ret === undefined ? null : ret;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormErrorPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormErrorPipe, isStandalone: false, name: "formError" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormErrorPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formError',
                    standalone: false,
                    pure: true,
                }]
        }] });

class FormHasErrorPipe {
    transform(meta, specificError, requireTouched = true, log = false) {
        const ret = meta?.hasError(specificError, requireTouched) ?? false;
        if (log) {
            console.log('hasFormError', meta, specificError, requireTouched, ret);
        }
        return ret;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormHasErrorPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormHasErrorPipe, isStandalone: false, name: "formHasError" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormHasErrorPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formHasError',
                    standalone: false,
                    pure: true,
                }]
        }] });

class FormMetaPipe {
    transform(form, path) {
        if (!form) {
            return of(null);
        }
        return form.hierarchy.getChild(path)?.meta$ ?? of(null);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormMetaPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormMetaPipe, isStandalone: false, name: "formMeta" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormMetaPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formMeta',
                    standalone: false,
                    pure: true,
                }]
        }] });

class FormArrayPipe {
    transform(form, path) {
        const ret = form.hierarchy.getChild(path);
        if (!ret) {
            throw new Error(`formArray: No child found at path "${path}"`);
        }
        if (ret.type !== 'array') {
            throw new Error(`formArray: Expected a FormArray at path "${path}", but got "${ret.type}"`);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return ret.control;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormArrayPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormArrayPipe, isStandalone: false, name: "formArray" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormArrayPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formArray',
                    standalone: false,
                }]
        }] });

class FormControlPipe {
    transform(form, path) {
        const ret = form.hierarchy.getChild(path);
        if (!ret) {
            throw new Error(`formControl: No child found at path "${path}"`);
        }
        if (ret.type !== 'control') {
            throw new Error(`formControl: Expected a FormControl at path "${path}", but got "${ret.type}"`);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return ret.control;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormControlPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormControlPipe, isStandalone: false, name: "formControl" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormControlPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formControl',
                    standalone: false,
                }]
        }] });

class FormGroupPipe {
    transform(form, path) {
        const ret = form.hierarchy.getChild(path);
        if (!ret) {
            throw new Error(`formGroup: No child found at path "${path}"`);
        }
        if (ret.type !== 'group') {
            throw new Error(`formGroup: Expected a FormGroup at path "${path}", but got "${ret.type}"`);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        console.log(ret.control);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return ret.control;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormGroupPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormGroupPipe, isStandalone: false, name: "formGroup" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormGroupPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formGroup',
                    standalone: false,
                }]
        }] });

class FormArrayValuePipe {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    transform(formRef, kind) {
        if (!formRef) {
            throw new Error('FormArrayValuePipe: formRef is null or undefined');
        }
        let ret;
        if (kind === 'standard') {
            ret = formRef.value$;
        }
        else {
            ret = formRef.rawValue$;
        }
        return ret;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormArrayValuePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormArrayValuePipe, isStandalone: false, name: "formArrayValue" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormArrayValuePipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formArrayValue',
                    standalone: false,
                    pure: true,
                }]
        }] });

class FormControlValuePipe {
    transform(group) {
        if (!group) {
            throw new Error('FormControlValuePipe: control is null or undefined');
        }
        return group.value$;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormControlValuePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormControlValuePipe, isStandalone: false, name: "formControlValue" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormControlValuePipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formControlValue',
                    standalone: false,
                    pure: true,
                }]
        }] });

class FormGroupValuePipe {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    transform(group, kind) {
        if (!group) {
            throw new Error('FormGroupValuePipe: group is null or undefined');
        }
        let ret;
        if (kind === 'standard') {
            ret = group.value$;
        }
        else {
            ret = group.rawValue$;
        }
        return ret;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormGroupValuePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormGroupValuePipe, isStandalone: false, name: "formGroupValue" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormGroupValuePipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formGroupValue',
                    standalone: false,
                    pure: true,
                }]
        }] });

class FormArrayControlsPipe {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    transform(form) {
        return form.controls$;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormArrayControlsPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: FormArrayControlsPipe, isStandalone: false, name: "formArrayControls" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: FormArrayControlsPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'formArrayControls',
                    standalone: false,
                }]
        }] });

class TableauUiFormModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormModule, declarations: [FormErrorPipe, FormHasErrorPipe, FormMetaPipe, FormArrayPipe, FormControlPipe, FormGroupPipe, FormArrayValuePipe, FormControlValuePipe, FormGroupValuePipe, FormArrayControlsPipe], imports: [ReactiveFormsModule, CommonModule], exports: [FormErrorPipe, FormHasErrorPipe, FormMetaPipe, FormArrayPipe, FormControlPipe, FormGroupPipe, FormArrayValuePipe, FormControlValuePipe, FormGroupValuePipe, FormArrayControlsPipe] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormModule, providers: [FB], imports: [ReactiveFormsModule, CommonModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiFormModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [ReactiveFormsModule, CommonModule],
                    declarations: [FormErrorPipe, FormHasErrorPipe, FormMetaPipe, FormArrayPipe, FormControlPipe, FormGroupPipe, FormArrayValuePipe, FormControlValuePipe, FormGroupValuePipe, FormArrayControlsPipe],
                    providers: [FB],
                    exports: [FormErrorPipe, FormHasErrorPipe, FormMetaPipe, FormArrayPipe, FormControlPipe, FormGroupPipe, FormArrayValuePipe, FormControlValuePipe, FormGroupValuePipe, FormArrayControlsPipe],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { FB, FormArrayControlsPipe, FormArrayPipe, FormArrayValuePipe, FormControlPipe, FormControlValuePipe, FormErrorPipe, FormGroupPipe, FormGroupValuePipe, FormHasErrorPipe, FormMetaPipe, TableauUiFormModule };
//# sourceMappingURL=tableau-ui-angular-form.mjs.map
