import * as i0 from '@angular/core';
import { inject, ElementRef, model, input, contentChild, viewChild, signal, ChangeDetectionStrategy, Component, effect, Directive, output, contentChildren, Pipe, HostListener, NgModule } from '@angular/core';
import { DialogService, LocalStackOptions, TableauUiDialogModule } from 'tableau-ui-angular/dialog';
import { PrefixComponent, SuffixComponent, TableauUiCommonModule } from 'tableau-ui-angular/common';
import * as i2$1 from 'tableau-ui-angular/utils';
import { generateRandomString, TableauUiUtilsModule } from 'tableau-ui-angular/utils';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i2 from 'tableau-ui-angular/icon';
import { TableauUiIconModule } from 'tableau-ui-angular/icon';

class MenuComponent {
    dialogService = inject(DialogService);
    elementRef = inject(ElementRef);
    /**
     * The parent control to which the autocomplete is attached to
     * // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
     */
    $parentControl = model();
    /**
     * The CSS text to apply to the dropdown container
     * @remarks
     * Use this to apply height, maxHeight, etc. to the dropdown container
     * @default '{}'
     */
    $menuContainerCss = model({
        marginTop: '-1px',
        background: 'var(--twc-color-base)',
        borderColor: 'var(--twc-color-border-light)',
        borderRadius: 'var(--twc-menu-border-radius)',
        borderStyle: 'solid',
        borderWidth: '1px',
        boxShadow: 'var(--twc-dialog-box-shadow)',
    }, {
        alias: 'menuContainerCss',
    });
    // The default CSS text to apply to the dropdown container. This is used to set the default values for the menuContainerCss property.
    defaultContainerCss = {
        outline: 'none',
        color: 'var(--twc-color-text)',
        lineHeight: 'normal',
        overFlowY: 'auto',
        overFlowX: 'hidden',
        userSelect: 'none',
    };
    /**
     * The CSS text to apply to the backdrop container
     * @remarks
     * Use this to apply user-select: none, etc. to the backdrop container
     * @default '{}'
     */
    $backdropCss = model({}, {
        alias: 'backdropCss',
    });
    /**
     * The width of the container
     * @remarks
     * Use 'parentWidth' to maintain the same width as the parent control
     * Use 'fit-content' to auto size the dropdown for the content
     * Use any CSS width value (1rem, 12px, etc) to explicitly set width
     * @default 'fit-content'
     */
    $width = input('fit-content', {
        alias: 'width',
    });
    /**
     * The max width of the container
     * @remarks
     * Keep it undefined for default behavior
     * Use any CSS width value (1rem, 12px, etc) to explicitly set maximum width
     * @default 'undefined'
     */
    $maxWidth = input(undefined, {
        alias: 'maxWidth',
    });
    /**
     * The height of the container
     * @remarks
     * Use 'fit-content' to auto size the dropdown for the content
     * Use any CSS height value (1rem, 12px, etc) to explicitly set height
     * @default 'fit-content'
     */
    $height = input('fit-content', {
        alias: 'height',
    });
    /**
     * The max height of the container
     * @remarks
     * Keep it undefined for default behavior
     * Use any CSS height value (1rem, 12px, etc) to explicitly set maximum height
     * @default 'undefined'
     */
    $maxHeight = input(undefined, {
        alias: 'maxHeight',
    });
    /**
     * Close on backdrop click
     * @remarks
     * When true, the menu will close when the backdrop is clicked
     * @default 'true'
     */
    $closeOnBackdropClick = input(true, {
        alias: 'closeOnBackdropClick',
    });
    /**
     * Close on escape key press
     * @remarks
     * When true, the menu will close when the escape key is pressed
     * @default 'true'
     */
    $closeOnEscape = input(true, {
        alias: 'closeOnEscape',
    });
    /**
     * Trap focus within the menu
     * @remarks
     * When true, the focus will be trapped within the menu until it is closed
     * @default 'true'
     */
    $trapFocus = input(true, {
        alias: 'trapFocus',
    });
    /**
     * The location of the menu relative to the parent control
     * @remarks
     * The menu may be repositioned if page bounds are hit to the opposite side
     * @default 'bottom'
     */
    $menuLocation = input('bottom', {
        alias: 'menuLocation',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $prefix = contentChild(PrefixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $suffix = contentChild(SuffixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $template = viewChild('dropdownTemplate');
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $openDialog = signal(undefined);
    open(forceReOpen = false) {
        const template = this.$template();
        if (!template) {
            throw new Error('Template is not set. Please set the template before opening the menu.');
        }
        const parentControl = this.$parentControl();
        if (!parentControl) {
            throw new Error('Parent control is not set. Please set the parent control before opening the menu.');
        }
        if (forceReOpen) {
            this.$openDialog()?.close();
            this.$openDialog.set(undefined);
        }
        if (this.$openDialog()) {
            return this.$openDialog();
        }
        const ref = this.dialogService.openTemplateDialog(template, {
            top: (_, actualHeight, parentRect) => {
                switch (this.$menuLocation()) {
                    case 'top': {
                        const val = parentRect.top - actualHeight;
                        if (val < 0 && parentRect.bottom + actualHeight < window.innerHeight) {
                            return `${parentRect.bottom}px`;
                        }
                        return `${parentRect.top - actualHeight}px`;
                    }
                    case 'bottom': {
                        const val = parentRect.bottom;
                        if (val + actualHeight > window.innerHeight && parentRect.top - actualHeight > 0) {
                            return `${parentRect.top - actualHeight}px`;
                        }
                        return `${val}px`;
                    }
                    case 'left':
                    case 'right': {
                        // the top parameter is the top parameter of the parent control by default
                        let { top } = parentRect;
                        // if it is higher than the availale window
                        if (top + actualHeight > window.innerHeight) {
                            top = window.innerHeight - actualHeight;
                        }
                        top = Math.max(top, 0);
                        return `${top}px`;
                    }
                    default: {
                        throw new Error(`Invalid menu location: ${this.$menuLocation()}`);
                    }
                }
            },
            left: (actualWidth, _, parentRect) => {
                switch (this.$menuLocation()) {
                    case 'left': {
                        const val = parentRect.left - actualWidth;
                        if (val < 0 && parentRect.right + actualWidth < window.innerWidth) {
                            return `${parentRect.right}px`;
                        }
                        return `${parentRect.left - actualWidth}px`;
                    }
                    case 'right': {
                        const val = parentRect.right;
                        if (val + actualWidth > window.innerWidth && parentRect.left - actualWidth > 0) {
                            return `${parentRect.left - actualWidth}px`;
                        }
                        return `${val}px`;
                    }
                    case 'top':
                    case 'bottom': {
                        // the left parameter is the left parameter of the parent control by default
                        let { left } = parentRect;
                        // if it is wider than the availale window
                        if (left + actualWidth > window.innerWidth) {
                            left = window.innerWidth - actualWidth;
                        }
                        left = Math.max(left, 0);
                        return `${left}px`;
                    }
                    default: {
                        throw new Error(`Invalid menu location: ${this.$menuLocation()}`);
                    }
                }
            },
            width: (parentRect) => {
                return this.$width() === 'parentWidth' ? `${parentRect.width}px` : this.$width();
            },
            closeOnBackdropClick: this.$closeOnBackdropClick(),
            closeOnEscape: this.$closeOnEscape(),
            trapFocus: this.$trapFocus(),
            height: this.$height(),
            maxHeight: this.$maxHeight(),
            maxWidth: this.$maxWidth(),
            containerCss: {
                ...this.defaultContainerCss,
                ...this.$menuContainerCss(),
            },
            backdropCss: this.$backdropCss(),
        }, null, new LocalStackOptions(parentControl.nativeElement));
        ref.closed$.subscribe(() => {
            if (this.$openDialog() === ref) {
                this.$openDialog.set(undefined);
            }
        });
        this.$openDialog.set(ref);
        return ref;
    }
    close() {
        this.$openDialog()?.close();
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: MenuComponent, isStandalone: false, selector: "tab-menu", inputs: { $parentControl: { classPropertyName: "$parentControl", publicName: "$parentControl", isSignal: true, isRequired: false, transformFunction: null }, $menuContainerCss: { classPropertyName: "$menuContainerCss", publicName: "menuContainerCss", isSignal: true, isRequired: false, transformFunction: null }, $backdropCss: { classPropertyName: "$backdropCss", publicName: "backdropCss", isSignal: true, isRequired: false, transformFunction: null }, $width: { classPropertyName: "$width", publicName: "width", isSignal: true, isRequired: false, transformFunction: null }, $maxWidth: { classPropertyName: "$maxWidth", publicName: "maxWidth", isSignal: true, isRequired: false, transformFunction: null }, $height: { classPropertyName: "$height", publicName: "height", isSignal: true, isRequired: false, transformFunction: null }, $maxHeight: { classPropertyName: "$maxHeight", publicName: "maxHeight", isSignal: true, isRequired: false, transformFunction: null }, $closeOnBackdropClick: { classPropertyName: "$closeOnBackdropClick", publicName: "closeOnBackdropClick", isSignal: true, isRequired: false, transformFunction: null }, $closeOnEscape: { classPropertyName: "$closeOnEscape", publicName: "closeOnEscape", isSignal: true, isRequired: false, transformFunction: null }, $trapFocus: { classPropertyName: "$trapFocus", publicName: "trapFocus", isSignal: true, isRequired: false, transformFunction: null }, $menuLocation: { classPropertyName: "$menuLocation", publicName: "menuLocation", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { $parentControl: "$parentControlChange", $menuContainerCss: "menuContainerCssChange", $backdropCss: "backdropCssChange" }, queries: [{ propertyName: "$prefix", first: true, predicate: PrefixComponent, descendants: true, isSignal: true }, { propertyName: "$suffix", first: true, predicate: SuffixComponent, descendants: true, isSignal: true }], viewQueries: [{ propertyName: "$template", first: true, predicate: ["dropdownTemplate"], descendants: true, isSignal: true }], ngImport: i0, template: "<ng-template #dropdownTemplate>\n    @if ($prefix()) {\n        <div class=\"options-prefix\">\n            <ng-content select=\"tab-prefix\" />\n        </div>\n    }\n    <ng-content />\n    @if ($suffix()) {\n        <div class=\"options-suffix\">\n            <ng-content select=\"tab-suffix\" />\n        </div>\n    }\n</ng-template>\n", styles: [":host{display:none}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-menu', standalone: false, changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-template #dropdownTemplate>\n    @if ($prefix()) {\n        <div class=\"options-prefix\">\n            <ng-content select=\"tab-prefix\" />\n        </div>\n    }\n    <ng-content />\n    @if ($suffix()) {\n        <div class=\"options-suffix\">\n            <ng-content select=\"tab-suffix\" />\n        </div>\n    }\n</ng-template>\n", styles: [":host{display:none}\n"] }]
        }] });

class MenuDirective {
    $menu = input.required({
        alias: 'menu',
    });
    el = inject(ElementRef);
    menuChanged = effect(() => {
        this.$menu().$parentControl.set(this.el);
    });
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "20.0.3", type: MenuDirective, isStandalone: false, selector: "[menu]", inputs: { $menu: { classPropertyName: "$menu", publicName: "menu", isSignal: true, isRequired: true, transformFunction: null } }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[menu]',
                    standalone: false,
                }]
        }] });

class MenuButtonComponent {
    id;
    contentElement = viewChild('buttonElement');
    $disabled = input(false, {
        alias: 'disabled',
    });
    $loading = input(false, {
        alias: 'loading',
    });
    $color = input('secondary', {
        alias: 'color',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $hoverToOpenSubMenuMs = input(undefined, {
        alias: 'hoverToOpenSubMenuMs',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $actualHoverMs = signal(undefined);
    updateChildrenHover = effect(() => {
        const hoverMs = this.$actualHoverMs();
        const children = this.$children();
        for (const child of children) {
            if (child.$hoverToOpenSubMenuMs() !== undefined) {
                child.$actualHoverMs.set(child.$hoverToOpenSubMenuMs());
            }
            else {
                child.$actualHoverMs.set(hoverMs);
            }
        }
    });
    // children = contentChildren(MenuButtonComponent);
    $highlight = signal(false);
    highlightEffect = effect(() => {
        this.highlightChange.emit(this.$highlight());
    });
    highlightChange = output();
    $children = contentChildren(MenuButtonComponent);
    mouseoverChange = output();
    $template = viewChild.required('template');
    openSubMenu = output();
    // eslint-disable-next-line @angular-eslint/no-output-native
    click = output();
    constructor() {
        this.id = generateRandomString();
    }
    hoverInterval = undefined;
    hoverstart = Date.now();
    onMouseEnter() {
        if (!this.$disabled()) {
            this.mouseoverChange.emit(true);
            const hoverMs = this.$actualHoverMs();
            if (hoverMs !== undefined && this.$children().length > 0) {
                this.hoverstart = Date.now();
                this.hoverInterval = setInterval(() => {
                    if (Date.now() - this.hoverstart > hoverMs) {
                        this.openSubMenu.emit(new Event('mouseenter'));
                        if (this.hoverInterval) {
                            clearInterval(this.hoverInterval);
                        }
                    }
                }, 50);
            }
        }
    }
    onMouseLeave() {
        if (this.hoverInterval) {
            clearInterval(this.hoverInterval);
        }
        this.$highlight.set(false);
    }
    ngOnDestroy() {
        if (this.hoverInterval) {
            clearInterval(this.hoverInterval);
        }
    }
    onMouseDown(e) {
        e.preventDefault();
        e.stopPropagation();
    }
    onClick(e) {
        if (this.$disabled()) {
            return;
        }
        if (this.$children().length > 0) {
            this.openSubMenu.emit(e);
            e.preventDefault();
            e.stopPropagation();
        }
        else {
            this.click.emit(e);
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuButtonComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: MenuButtonComponent, isStandalone: false, selector: "tab-menu-button", inputs: { $disabled: { classPropertyName: "$disabled", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null }, $loading: { classPropertyName: "$loading", publicName: "loading", isSignal: true, isRequired: false, transformFunction: null }, $color: { classPropertyName: "$color", publicName: "color", isSignal: true, isRequired: false, transformFunction: null }, $hoverToOpenSubMenuMs: { classPropertyName: "$hoverToOpenSubMenuMs", publicName: "hoverToOpenSubMenuMs", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { highlightChange: "highlightChange", mouseoverChange: "mouseoverChange", openSubMenu: "openSubMenu", click: "click" }, queries: [{ propertyName: "$children", predicate: MenuButtonComponent, isSignal: true }], viewQueries: [{ propertyName: "contentElement", first: true, predicate: ["buttonElement"], descendants: true, isSignal: true }, { propertyName: "$template", first: true, predicate: ["template"], descendants: true, isSignal: true }], ngImport: i0, template: `
        <ng-template #template>
            <div
                class="menu-button"
                role="button"
                type="button"
                tabindex="-1"
                [ngClass]="$color()"
                [class.loading]="$loading()"
                [class.highlight]="$highlight()"
                [attr.disabled]="$disabled() || $loading() ? true : false"
                [id]="id"
                (mouseenter)="onMouseEnter()"
                (mouseleave)="onMouseLeave()"
                (mousedown)="onMouseDown($event)"
                (click)="onClick($event)"
            >
                @if ($loading()) {
                    <span class="loader-wrapper">
                        <img
                            alt="Wait indicator"
                            class="f79w7hb"
                            src="data:image/svg+xml,%0A%3Csvg%20viewBox%3D%220%200%2050%2050%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%0A%20%20%3Cstyle%3E%0A%20%20%20%20%23tail%20%7B%20fill%3A%20url(%23fade)%20%7D%0A%20%20%20%20%23head%20%7B%20fill%3A%20rgb(97%2C%20101%2C%20112)%20%7D%0A%20%20%20%20stop%20%7B%20stop-color%3A%20rgb(97%2C%20101%2C%20112)%20%7D%0A%20%20%3C%2Fstyle%3E%0A%20%20%3ClinearGradient%20id%3D%22fade%22%20x2%3D%2250%22%20y1%3D%2225%22%20y2%3D%2225%22%20gradientUnits%3D%22userSpaceOnUse%22%3E%0A%20%20%20%20%3Cstop%20offset%3D%220%22%20stop-opacity%3D%220%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.15%22%20stop-opacity%3D%22.04%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.3%22%20stop-opacity%3D%22.16%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.45%22%20stop-opacity%3D%22.36%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.61%22%20stop-opacity%3D%22.64%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.76%22%2F%3E%0A%20%20%3C%2FlinearGradient%3E%0A%20%20%3Cpath%20id%3D%22head%22%20d%3D%22M0%2025a25%2025%200%201%200%2050%200h-3.9a21.1%2021.1%200%201%201-42.2%200%22%20%2F%3E%0A%20%20%3Cpath%20id%3D%22tail%22%20d%3D%22M50%2025a25%2025%200%200%200-50%200h3.9a21.1%2021.1%200%201%201%2042.2%200%22%20%2F%3E%0A%3C%2Fsvg%3E"
                        />
                    </span>
                }
                <div class="button-content"><ng-content /></div>
                @if ($children().length > 0) {
                    <tab-icon class="expand" value="keyboard_arrow_right" />
                }
            </div>
        </ng-template>
    `, isInline: true, styles: [".menu-button{-webkit-user-select:none;user-select:none;box-sizing:border-box;pointer-events:all;cursor:pointer;font-family:inherit;font-size:var(--twc-font-size-body);font-weight:700;height:fit-content;min-height:2em;line-height:1.5;min-width:1rem;padding:0;align-items:center;text-decoration:none;display:grid;grid-template-columns:1fr auto;gap:.25rem;width:100%;background-color:transparent;border:none;padding-left:.5rem;padding-right:.5rem;white-space:nowrap}.menu-button.has-child{padding-right:0}.menu-button.loading{grid-template-columns:auto 1fr auto}.menu-button .loader-wrapper{grid-column:1;visibility:hidden;display:flex}@keyframes loading-button-spinner{to{transform:rotate(360deg)}}.menu-button .loader-wrapper img{animation:loading-button-spinner 1s linear infinite;cursor:wait;margin:auto;overflow:hidden;width:1.1rem;height:1.1rem}.menu-button .button-content{display:flex;align-items:center}.menu-button .expand{font-size:1.2rem}.menu-button[disabled]:not([disabled=false]){color:var(--twc-color-disabled)!important;cursor:default;pointer-events:none}.menu-button.primary{color:var(--twc-color-primary)}.menu-button.secondary{color:var(--twc-color-text)}.menu-button.error{color:var(--twc-color-error)}.menu-button.highlight{background-color:var(--twc-color-highlight);border:none;outline:none}\n"], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: i2.IconComponent, selector: "tab-icon", inputs: ["value", "color", "type", "fill", "weight", "grade", "opticalSizePx"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuButtonComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-menu-button', standalone: false, template: `
        <ng-template #template>
            <div
                class="menu-button"
                role="button"
                type="button"
                tabindex="-1"
                [ngClass]="$color()"
                [class.loading]="$loading()"
                [class.highlight]="$highlight()"
                [attr.disabled]="$disabled() || $loading() ? true : false"
                [id]="id"
                (mouseenter)="onMouseEnter()"
                (mouseleave)="onMouseLeave()"
                (mousedown)="onMouseDown($event)"
                (click)="onClick($event)"
            >
                @if ($loading()) {
                    <span class="loader-wrapper">
                        <img
                            alt="Wait indicator"
                            class="f79w7hb"
                            src="data:image/svg+xml,%0A%3Csvg%20viewBox%3D%220%200%2050%2050%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%0A%20%20%3Cstyle%3E%0A%20%20%20%20%23tail%20%7B%20fill%3A%20url(%23fade)%20%7D%0A%20%20%20%20%23head%20%7B%20fill%3A%20rgb(97%2C%20101%2C%20112)%20%7D%0A%20%20%20%20stop%20%7B%20stop-color%3A%20rgb(97%2C%20101%2C%20112)%20%7D%0A%20%20%3C%2Fstyle%3E%0A%20%20%3ClinearGradient%20id%3D%22fade%22%20x2%3D%2250%22%20y1%3D%2225%22%20y2%3D%2225%22%20gradientUnits%3D%22userSpaceOnUse%22%3E%0A%20%20%20%20%3Cstop%20offset%3D%220%22%20stop-opacity%3D%220%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.15%22%20stop-opacity%3D%22.04%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.3%22%20stop-opacity%3D%22.16%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.45%22%20stop-opacity%3D%22.36%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.61%22%20stop-opacity%3D%22.64%22%2F%3E%0A%20%20%20%20%3Cstop%20offset%3D%22.76%22%2F%3E%0A%20%20%3C%2FlinearGradient%3E%0A%20%20%3Cpath%20id%3D%22head%22%20d%3D%22M0%2025a25%2025%200%201%200%2050%200h-3.9a21.1%2021.1%200%201%201-42.2%200%22%20%2F%3E%0A%20%20%3Cpath%20id%3D%22tail%22%20d%3D%22M50%2025a25%2025%200%200%200-50%200h3.9a21.1%2021.1%200%201%201%2042.2%200%22%20%2F%3E%0A%3C%2Fsvg%3E"
                        />
                    </span>
                }
                <div class="button-content"><ng-content /></div>
                @if ($children().length > 0) {
                    <tab-icon class="expand" value="keyboard_arrow_right" />
                }
            </div>
        </ng-template>
    `, changeDetection: ChangeDetectionStrategy.OnPush, styles: [".menu-button{-webkit-user-select:none;user-select:none;box-sizing:border-box;pointer-events:all;cursor:pointer;font-family:inherit;font-size:var(--twc-font-size-body);font-weight:700;height:fit-content;min-height:2em;line-height:1.5;min-width:1rem;padding:0;align-items:center;text-decoration:none;display:grid;grid-template-columns:1fr auto;gap:.25rem;width:100%;background-color:transparent;border:none;padding-left:.5rem;padding-right:.5rem;white-space:nowrap}.menu-button.has-child{padding-right:0}.menu-button.loading{grid-template-columns:auto 1fr auto}.menu-button .loader-wrapper{grid-column:1;visibility:hidden;display:flex}@keyframes loading-button-spinner{to{transform:rotate(360deg)}}.menu-button .loader-wrapper img{animation:loading-button-spinner 1s linear infinite;cursor:wait;margin:auto;overflow:hidden;width:1.1rem;height:1.1rem}.menu-button .button-content{display:flex;align-items:center}.menu-button .expand{font-size:1.2rem}.menu-button[disabled]:not([disabled=false]){color:var(--twc-color-disabled)!important;cursor:default;pointer-events:none}.menu-button.primary{color:var(--twc-color-primary)}.menu-button.secondary{color:var(--twc-color-text)}.menu-button.error{color:var(--twc-color-error)}.menu-button.highlight{background-color:var(--twc-color-highlight);border:none;outline:none}\n"] }]
        }], ctorParameters: () => [] });

class EntriesPipe {
    transform(arr) {
        return [...arr.entries()];
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: EntriesPipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: EntriesPipe, isStandalone: false, name: "entries" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: EntriesPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'entries',
                    standalone: false,
                }]
        }] });

class MenuButtonGroupComponent {
    nativeElement = inject(ElementRef);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $hoverToOpenSubMenuMs = input(500, {
        alias: 'hoverToOpenSubMenuMs',
    });
    buttonClicked = output();
    $children = contentChildren(MenuButtonComponent);
    previousChildrenIds = [];
    childrenEffect = effect(() => {
        const children = this.$children();
        const ids = children.map((c) => c.id);
        const addedIds = ids.filter((id) => !this.previousChildrenIds.includes(id));
        const removedIds = this.previousChildrenIds.filter((id) => !ids.includes(id));
        if (!addedIds.length && !removedIds.length) {
            return;
        }
        this.previousChildrenIds = ids;
        const hoverMs = this.$hoverToOpenSubMenuMs();
        for (const child of children) {
            if (child.$hoverToOpenSubMenuMs() !== undefined) {
                child.$actualHoverMs.set(child.$hoverToOpenSubMenuMs());
            }
            else {
                child.$actualHoverMs.set(hoverMs);
            }
        }
        this.init();
    });
    $menuGroupStack = signal([]);
    $height = signal(0);
    $width = signal(0);
    init() {
        this.destroyGroupsUntil();
        this.addMenuGroup();
    }
    ngOnDestroy() {
        this.destroyGroupsUntil();
    }
    addMenuGroup(parentButton) {
        const children = parentButton?.$children() ?? this.$children();
        if (children.length === 0) {
            return undefined;
        }
        const group = {
            id: generateRandomString(),
            // position is fixed and off screen
            // this is used to calculate actual size of the menu group
            $top: signal(-10000),
            $left: signal(-10000),
            element: null,
            $position: signal('fixed'),
            buttons: children,
            mouseoverSubscriptions: children.map((c) => c.mouseoverChange.subscribe((mouseOver) => {
                for (const child of children) {
                    child.$highlight.set(false);
                }
                if (mouseOver) {
                    this.nativeElement.nativeElement.focus();
                    c.$highlight.set(true);
                }
            })),
            openSubMenuSubscriptions: children
                .filter((e) => e.$children().length > 0)
                .map((c) => c.openSubMenu.subscribe(() => {
                if (this.$menuGroupStack().some((s) => s.parentButton?.id === c.id)) {
                    return;
                }
                const g = this.addMenuGroup(c);
                const firstButton = g?.buttons.find((b) => !b.$disabled());
                if (firstButton) {
                    firstButton.$highlight.set(true);
                }
            })),
            clickSubscriptions: children.map((c) => c.click.subscribe((e) => {
                const stack = this.$menuGroupStack();
                if (stack.length > 0) {
                    const firstGroup = stack[0];
                    this.destroyGroupsUntil(firstGroup.id);
                }
                this.buttonClicked.emit({
                    button: c,
                    event: e,
                });
            })),
            highlightSubscriptions: children.map((c) => c.highlightChange.subscribe((highlight) => {
                if (highlight) {
                    // if highlight changed in this group, make sure it's the last group in the list
                    this.destroyGroupsUntil(group.id);
                }
            })),
            parentButton: parentButton?.id !== undefined ? document.getElementById(parentButton.id) : null,
        };
        this.$menuGroupStack.update((groups) => {
            const newGroups = [...groups, group];
            return newGroups;
        });
        // wait for our new group to render (max 10 seconds)
        this.updateSizes();
        return group;
    }
    updateSizes() {
        const groups = this.$menuGroupStack();
        const currentRect = this.nativeElement.nativeElement.getBoundingClientRect();
        let minTop = 0;
        let minLeft = 0;
        let maxTop = 0;
        let maxLeft = 0;
        for (const group of groups) {
            if (!group.element) {
                continue;
            }
            const rect = group.element.getBoundingClientRect();
            const elWidth = rect.width;
            const elHeight = rect.height;
            let top;
            let left;
            if (!group.parentButton) {
                top = 0;
                left = 0;
                group.$position.set('absolute');
                minTop = Math.min(minTop, 0);
                minLeft = Math.min(minLeft, 0);
                maxTop = Math.max(maxTop, elHeight);
                maxLeft = Math.max(maxLeft, elWidth);
            }
            else {
                const parentRect = group.parentButton.getBoundingClientRect();
                top = parentRect.top - currentRect.top;
                minTop = Math.min(minTop, top);
                left = parentRect.right;
                if (left + rect.width > window.innerWidth) {
                    left = parentRect.left - rect.width;
                }
                left = left - currentRect.left;
                minLeft = Math.min(minLeft, left);
                maxTop = Math.max(maxTop, top + elHeight);
                maxLeft = Math.max(maxLeft, left + elWidth);
                group.$position.set('absolute');
            }
            group.$top.set(top);
            group.$left.set(left);
        }
        this.$height.set(maxTop - minTop);
        this.$width.set(maxLeft - minLeft);
    }
    destroyGroupsUntil(id) {
        const groups = this.$menuGroupStack();
        const curIndex = groups.findIndex((g) => g.id === id);
        const removed = groups.splice(curIndex + 1);
        for (const group of removed) {
            group.mouseoverSubscriptions.forEach((s) => {
                s.unsubscribe();
            });
            group.openSubMenuSubscriptions.forEach((s) => {
                s.unsubscribe();
            });
            group.highlightSubscriptions.forEach((s) => {
                s.unsubscribe();
            });
            group.clickSubscriptions.forEach((s) => {
                s.unsubscribe();
            });
            group.buttons.forEach((b) => {
                b.$highlight.set(false);
            });
            // group.element?.remove();
        }
        // this will also remove from DOM
        this.$menuGroupStack.set(groups);
        this.updateSizes();
    }
    onFocusIn() {
        // clear everything except the first group in the stack
        const groupStack = this.$menuGroupStack().slice(0, 1);
        // highlight the first button in the group
        const firstButton = groupStack[0].buttons.find((b) => !b.$disabled());
        if (firstButton) {
            firstButton.$highlight.set(true);
        }
    }
    onFocusOut() {
        // clear everything except the first group in the stack
        const groupStack = this.$menuGroupStack();
        if (groupStack.length > 0) {
            const rootId = groupStack[0].id;
            this.destroyGroupsUntil(rootId);
            for (const b of groupStack[0].buttons) {
                b.$highlight.set(false);
            }
        }
    }
    onKeyDown(event) {
        if (event.key === 'ArrowUp' || event.key === 'ArrowDown') {
            // we only handle navitation on top of the stack
            const stack = this.$menuGroupStack();
            const buttons = stack[stack.length - 1]?.buttons;
            let currentIndex = -1;
            currentIndex = buttons.findIndex((child) => child.$highlight());
            let nextIndex;
            if (event.key === 'ArrowDown') {
                if (currentIndex === -1) {
                    // find the first non disabled option
                    nextIndex = buttons.findIndex((e) => !e.$disabled());
                }
                else {
                    // find the next option that is not disabled
                    nextIndex = buttons.findIndex((o, i) => i > currentIndex && !o.$disabled());
                    // if no option is found, find the next option that is not disabled before the current item
                    if (nextIndex === -1) {
                        nextIndex = buttons.findIndex((o, i) => i < currentIndex && !o.$disabled());
                    }
                }
            }
            else if (event.key === 'ArrowUp') {
                if (currentIndex === -1) {
                    // find the last non disabled option
                    nextIndex = buttons
                        .slice()
                        .reverse()
                        .findIndex((o) => !o.$disabled());
                    if (nextIndex !== -1) {
                        nextIndex = buttons.length - nextIndex - 1;
                    }
                }
                else {
                    const flippedCurrentIndex = buttons.length - currentIndex - 1;
                    // find the next option that is not disabled
                    nextIndex = [...buttons].reverse().findIndex((o, i) => i > flippedCurrentIndex && !o.$disabled());
                    // if no option is found, find the next option that is not disabled before the current item
                    if (nextIndex === -1) {
                        nextIndex = [...buttons].reverse().findIndex((o, i) => i < flippedCurrentIndex && !o.$disabled());
                    }
                    if (nextIndex !== -1) {
                        nextIndex = buttons.length - nextIndex - 1;
                    }
                }
            }
            else {
                nextIndex = -1;
            }
            for (const [i, b] of buttons.entries()) {
                b.$highlight.set(i === nextIndex);
            }
        }
        if (event.key === 'Enter' || event.key === ' ' || event.key === 'ArrowRight') {
            const highlightedButton = this.$menuGroupStack()[this.$menuGroupStack().length - 1]?.buttons?.find((b) => b.$highlight());
            if (!highlightedButton || highlightedButton.$disabled()) {
                return;
            }
            if (highlightedButton.$children().length > 0) {
                highlightedButton.onClick(event);
            }
            else if (event.key === 'Enter' || event.key === ' ') {
                highlightedButton.onClick(event);
            }
        }
        if (event.key === 'ArrowLeft') {
            if (this.$menuGroupStack().length > 1) {
                const parentGroupId = this.$menuGroupStack()[this.$menuGroupStack().length - 2].id;
                this.destroyGroupsUntil(parentGroupId);
            }
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuButtonGroupComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: MenuButtonGroupComponent, isStandalone: false, selector: "tab-menu-button-group", inputs: { $hoverToOpenSubMenuMs: { classPropertyName: "$hoverToOpenSubMenuMs", publicName: "hoverToOpenSubMenuMs", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { buttonClicked: "buttonClicked" }, host: { attributes: { "tabindex": "0" }, listeners: { "focusin": "onFocusIn()", "focusout": "onFocusOut()", "keydown": "onKeyDown($event)" }, properties: { "style.minHeight": "$height() + \"px\"", "style.minWidth": "$width() + \"px\"" } }, queries: [{ propertyName: "$children", predicate: MenuButtonComponent, isSignal: true }], ngImport: i0, template: `
        @for (entry of $menuGroupStack() | entries; track entry[1].id) {
            <!-- eslint-disable @angular-eslint/template/prefer-template-literal -->
            <div
                class="menu-group"
                resizeWatcher
                [id]="entry[1].id"
                [style.position]="entry[1].$position()"
                [style.top]="entry[1].$top() + 'px'"
                [style.left]="entry[1].$left() + 'px'"
                (resized)="entry[1].element = $event.currentElement.nativeElement; updateSizes()"
            >
                @for (btnEntry of entry[1].buttons | entries; track btnEntry[0]) {
                    <ng-container *ngTemplateOutlet="btnEntry[1].$template()" />
                }
            </div>
        }
    `, isInline: true, styles: [":host{outline:none!important;position:relative;display:block;width:100%;-webkit-user-select:none;user-select:none}\n", ".menu-group{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow-y:auto;overflow-x:hidden;-webkit-user-select:none;user-select:none}\n"], dependencies: [{ kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i2$1.ResizeWatcherDirective, selector: "[resizeWatcher]", outputs: ["resized"] }, { kind: "pipe", type: EntriesPipe, name: "entries" }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MenuButtonGroupComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-menu-button-group', standalone: false, template: `
        @for (entry of $menuGroupStack() | entries; track entry[1].id) {
            <!-- eslint-disable @angular-eslint/template/prefer-template-literal -->
            <div
                class="menu-group"
                resizeWatcher
                [id]="entry[1].id"
                [style.position]="entry[1].$position()"
                [style.top]="entry[1].$top() + 'px'"
                [style.left]="entry[1].$left() + 'px'"
                (resized)="entry[1].element = $event.currentElement.nativeElement; updateSizes()"
            >
                @for (btnEntry of entry[1].buttons | entries; track btnEntry[0]) {
                    <ng-container *ngTemplateOutlet="btnEntry[1].$template()" />
                }
            </div>
        }
    `, changeDetection: ChangeDetectionStrategy.OnPush, host: {
                        tabindex: '0',
                        '[style.minHeight]': '$height() + "px"',
                        '[style.minWidth]': '$width() + "px"',
                    }, styles: [":host{outline:none!important;position:relative;display:block;width:100%;-webkit-user-select:none;user-select:none}\n", ".menu-group{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow-y:auto;overflow-x:hidden;-webkit-user-select:none;user-select:none}\n"] }]
        }], propDecorators: { onFocusIn: [{
                type: HostListener,
                args: ['focusin']
            }], onFocusOut: [{
                type: HostListener,
                args: ['focusout']
            }], onKeyDown: [{
                type: HostListener,
                args: ['keydown', ['$event']]
            }] } });

class ButtonMenuComponent extends MenuComponent {
    $menuContainerCss = model({
        pointerEvents: 'none',
        marginTop: '-1px',
    }, {
        alias: 'menuContainerCss',
    });
    $closeOnBackdropClick = model(true, {
        alias: 'closeOnBackdropClick',
    });
    $width = model('fit-content', {
        alias: 'width',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    menuGroup = contentChild(MenuButtonGroupComponent);
    subs = [];
    open(forceReOpen = false) {
        const ref = super.open(forceReOpen);
        if (ref && this.menuGroup()) {
            window.requestAnimationFrame(() => this.menuGroup()?.nativeElement?.nativeElement?.focus());
            const sub = this.menuGroup()?.buttonClicked.subscribe(() => {
                ref.close();
            });
            if (sub) {
                this.subs.push(sub);
            }
        }
        return ref;
    }
    ngOnDestroy() {
        this.subs.forEach((sub) => {
            sub.unsubscribe();
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: ButtonMenuComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: ButtonMenuComponent, isStandalone: false, selector: "tab-button-menu", inputs: { $menuContainerCss: { classPropertyName: "$menuContainerCss", publicName: "menuContainerCss", isSignal: true, isRequired: false, transformFunction: null }, $closeOnBackdropClick: { classPropertyName: "$closeOnBackdropClick", publicName: "closeOnBackdropClick", isSignal: true, isRequired: false, transformFunction: null }, $width: { classPropertyName: "$width", publicName: "width", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { $menuContainerCss: "menuContainerCssChange", $closeOnBackdropClick: "closeOnBackdropClickChange", $width: "widthChange" }, queries: [{ propertyName: "menuGroup", first: true, predicate: MenuButtonGroupComponent, descendants: true, isSignal: true }], usesInheritance: true, ngImport: i0, template: "<ng-template #dropdownTemplate>\n    @if ($prefix()) {\n        <div class=\"options-prefix\">\n            <ng-content select=\"tab-prefix\" />\n        </div>\n    }\n    <ng-content />\n    @if ($suffix()) {\n        <div class=\"options-suffix\">\n            <ng-content select=\"tab-suffix\" />\n        </div>\n    }\n</ng-template>\n", styles: [":host{display:none}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: ButtonMenuComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-button-menu', standalone: false, changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-template #dropdownTemplate>\n    @if ($prefix()) {\n        <div class=\"options-prefix\">\n            <ng-content select=\"tab-prefix\" />\n        </div>\n    }\n    <ng-content />\n    @if ($suffix()) {\n        <div class=\"options-suffix\">\n            <ng-content select=\"tab-suffix\" />\n        </div>\n    }\n</ng-template>\n", styles: [":host{display:none}\n"] }]
        }] });

class TableauUiMenuModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMenuModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMenuModule, declarations: [MenuComponent, MenuDirective, MenuButtonComponent, MenuButtonGroupComponent, ButtonMenuComponent, EntriesPipe], imports: [CommonModule, TableauUiCommonModule, TableauUiDialogModule, TableauUiIconModule, TableauUiUtilsModule], exports: [MenuComponent, MenuDirective, MenuButtonComponent, MenuButtonGroupComponent, ButtonMenuComponent] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMenuModule, imports: [CommonModule, TableauUiCommonModule, TableauUiDialogModule, TableauUiIconModule, TableauUiUtilsModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMenuModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, TableauUiCommonModule, TableauUiDialogModule, TableauUiIconModule, TableauUiUtilsModule],
                    declarations: [MenuComponent, MenuDirective, MenuButtonComponent, MenuButtonGroupComponent, ButtonMenuComponent, EntriesPipe],
                    exports: [MenuComponent, MenuDirective, MenuButtonComponent, MenuButtonGroupComponent, ButtonMenuComponent],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { ButtonMenuComponent, MenuButtonComponent, MenuButtonGroupComponent, MenuComponent, MenuDirective, TableauUiMenuModule };
//# sourceMappingURL=tableau-ui-angular-menu.mjs.map
