import * as i0 from '@angular/core';
import { inject, input, model, output, ElementRef, effect, untracked, forwardRef, ChangeDetectionStrategy, Component, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { ThemeService } from 'tableau-ui-angular/theme';
import { CommonModule } from '@angular/common';

let monacoLoadPromise = undefined;
class MonacoEditorComponent {
    _editor;
    themeService = inject(ThemeService);
    // #region Inputs and Outputs
    /**
     * Sets the options for the monaco editor, such scrollbars, minimap, etc
     * @default {}
     */
    $options = input({}, {
        alias: 'options',
    });
    /**
     * The URI of the current document.
     * This is used to identify the model in monaco.
     */
    $uri = input.required({
        alias: 'uri',
    });
    /**
     * The language of the current document.
     * This is used to set the language of the model in monaco.
     * For example, 'javascript', 'typescript', 'html', etc.
     */
    $language = input.required({
        alias: 'language',
    });
    /**
     * The value of the current document.
     * This can do two-way binding with the editor.
     */
    $value = model('', {
        alias: 'value',
    });
    /**
     * The theme of the monaco editor.
     * Can be 'auto', 'vs', 'vs-dark', 'hc-black', or 'hc-light'.
     * 'auto' will use the current library palette theme.
     * @default 'auto'
     */
    $theme = input('auto', {
        alias: 'theme',
    });
    /**
     * Only settable once.
     * Defines a function that is called when monaco is ready and before component init.
     * Use this to set language diagnostics, etc.
     */
    $onMonacoLoad = input(undefined, {
        alias: 'onMonacoLoad',
    });
    /**
     * If true, the model will be disposed when the component is destroyed.
     * This is useful to avoid memory leaks when the editor is not used anymore.
     * @default true
     */
    $disposeModelOnDestroy = input(true, {
        alias: 'disposeModelOnDestroy',
    });
    /**
     * Output event that is emitted when the editor is loaded.
     */
    editorLoaded = output();
    // #endregion Inputs and Outputs
    editorContainer = inject(ElementRef);
    // #region Effects
    // When the option changes, re-initialize the entire editor
    optionsChanged = effect(() => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const options = this.$options(); // need this for the effect to trigger
        if (this._editor) {
            this._editor.dispose();
            this._editor = undefined;
            this.initEditor();
        }
    });
    // When the URI changes, update the editor to use the new URI if it's initialized
    uriChanged = effect(() => {
        const uri = this.$uri();
        const language = untracked(() => this.$language());
        const value = untracked(() => this.$value());
        if (this._editor) {
            // update or create the model with the new URI
            const editorModel = this.updateOrCreateModel(monaco.Uri.parse(uri), language, value);
            this._editor.setModel(editorModel);
        }
    });
    // When the language changes, update the editor to use the new language if it's initialized
    languageChanged = effect(() => {
        const language = this.$language();
        const uri = untracked(() => this.$uri());
        if (this._editor) {
            // update or create the model with the new language
            const editorModel = monaco.editor.getModel(monaco.Uri.parse(uri));
            if (!editorModel) {
                console.warn('Can not update language. Monaco editor model not found for URI:', uri);
            }
            else {
                monaco.editor.setModelLanguage(editorModel, language);
            }
            this._editor.setModel(editorModel);
        }
    });
    // When the value changes, update the editor to use the new value if it's initialized
    valueChanged = effect(() => {
        const value = this.$value();
        const uri = untracked(() => this.$uri());
        if (this._editor) {
            // update or create the model with the new value
            const editorModel = monaco.editor.getModel(monaco.Uri.parse(uri));
            if (!editorModel) {
                console.warn('Can not update value. Monaco editor model not found for URI:', uri);
            }
            else {
                editorModel.setValue(value);
            }
        }
    });
    // When the theme changes, update the editor to use the new theme if it's initialized
    themeChanged = effect(() => {
        const themeInput = this.$theme();
        const systemTheme = this.themeService.$appliedThemeMode();
        if (this._editor) {
            const theme = this.getMonacoTheme(themeInput, systemTheme);
            monaco.editor.setTheme(theme);
        }
    });
    // #endregion Effects
    ngAfterViewInit() {
        // if monaco already started loading, init the editor when it is loaded
        if (monacoLoadPromise) {
            void monacoLoadPromise.then(() => {
                this.initEditor();
            });
            return;
        }
        monacoLoadPromise = new Promise((resolve) => {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
            if (typeof window.monaco === 'object') {
                this.initEditor();
                resolve();
                return;
            }
            const baseUrl = (document.head.getElementsByTagName('base').item(0)?.href ?? window.location.origin + '/') + 'monaco-edit/vs';
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const onGotAmdLoader = (require) => {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
                const usedRequire = require ?? window.require;
                const requireConfig = { paths: { vs: baseUrl } };
                // Load monaco
                // eslint-disable-next-line @typescript-eslint/no-unsafe-call, @typescript-eslint/no-unsafe-member-access
                usedRequire.config(requireConfig);
                // eslint-disable-next-line @typescript-eslint/no-unsafe-call
                usedRequire([`vs/editor/editor.main`], () => {
                    this.initEditor();
                    resolve();
                });
            };
            // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions, @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
            if (!window.require) {
                const loaderScript = document.createElement('script');
                loaderScript.type = 'text/javascript';
                loaderScript.src = `${baseUrl}/loader.js`;
                loaderScript.addEventListener('load', () => {
                    onGotAmdLoader();
                });
                document.body.appendChild(loaderScript);
                // Load AMD loader without over-riding node's require
                // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions, @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
            }
            else if (!window.require.config) {
                const src = `${baseUrl}/loader.js`;
                const loaderRequest = new XMLHttpRequest();
                loaderRequest.addEventListener('load', () => {
                    const scriptElem = document.createElement('script');
                    scriptElem.type = 'text/javascript';
                    scriptElem.text = [
                        // Monaco uses a custom amd loader that over-rides node's require.
                        // Keep a reference to node's require so we can restore it after executing the amd loader file.
                        'var nodeRequire = require;',
                        loaderRequest.responseText.replace('"use strict";', ''),
                        // Save Monaco's amd require and restore Node's require
                        'var monacoAmdRequire = require;',
                        'require = nodeRequire;',
                        'require.nodeRequire = require;',
                    ].join('\n');
                    document.body.appendChild(scriptElem);
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
                    onGotAmdLoader(window.monacoAmdRequire);
                });
                loaderRequest.open('GET', src);
                loaderRequest.send();
            }
            else {
                onGotAmdLoader();
            }
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    onPropagateChange = () => { };
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    onPropagateTouched = () => { };
    // #region ControlValueAccessor
    writeValue(val) {
        this.$value.set(val);
    }
    registerOnChange(fn) {
        this.onPropagateChange = fn;
    }
    registerOnTouched(fn) {
        this.onPropagateTouched = fn;
    }
    // #endregion ControlValueAccessor
    resizeObserver;
    initEditor() {
        this.destroyResizeObserver();
        const onLoad = this.$onMonacoLoad();
        if (onLoad) {
            onLoad(monaco);
        }
        // update or create the model
        const editorModel = this.updateOrCreateModel(monaco.Uri.parse(this.$uri()), this.$language(), this.$value());
        this._editor = monaco.editor.create(this.editorContainer.nativeElement, { ...this.$options(), model: editorModel });
        monaco.editor.setTheme(this.getMonacoTheme(this.$theme(), this.themeService.$appliedThemeMode()));
        // this._editor.setValue(this._value);
        this._editor.onDidChangeModelContent(() => {
            if (!this._editor) {
                return;
            }
            const value = this._editor.getValue() ?? '';
            this.$value.set(value);
            this.onPropagateChange(value);
        });
        this._editor.onDidBlurEditorText(() => {
            this.onPropagateTouched();
            if (this._editor) {
                this._editor.setSelection(new monaco.Selection(1, 1, 1, 1));
            }
        });
        this.resizeObserver = new ResizeObserver(() => {
            if (this._editor) {
                this._editor.layout();
            }
        });
        this.resizeObserver.observe(this.editorContainer.nativeElement);
        this.editorLoaded.emit(this._editor);
    }
    updateOrCreateModel(uri, language, value) {
        let editorModel = monaco.editor.getModel(uri);
        if (editorModel) {
            editorModel.setValue(value);
            if (editorModel.getLanguageId() !== language) {
                monaco.editor.setModelLanguage(editorModel, language);
            }
        }
        else {
            editorModel = monaco.editor.createModel(value, language, uri);
        }
        return editorModel;
    }
    getMonacoTheme(themeInput, systemTheme) {
        if (themeInput === 'auto') {
            return systemTheme === 'dark' ? 'vs-dark' : 'vs';
        }
        return themeInput;
    }
    destroyResizeObserver() {
        if (this.resizeObserver) {
            this.resizeObserver.disconnect();
            this.resizeObserver = undefined;
        }
    }
    ngOnDestroy() {
        this.destroyResizeObserver();
        if (this._editor) {
            const editorModel = this._editor.getModel();
            if (editorModel != null && this.$disposeModelOnDestroy()) {
                // Dispose the model to avoid memory leaks
                editorModel.dispose();
            }
            this._editor.dispose();
            this._editor = undefined;
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MonacoEditorComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "20.0.3", type: MonacoEditorComponent, isStandalone: false, selector: "tab-monaco-editor", inputs: { $options: { classPropertyName: "$options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null }, $uri: { classPropertyName: "$uri", publicName: "uri", isSignal: true, isRequired: true, transformFunction: null }, $language: { classPropertyName: "$language", publicName: "language", isSignal: true, isRequired: true, transformFunction: null }, $value: { classPropertyName: "$value", publicName: "value", isSignal: true, isRequired: false, transformFunction: null }, $theme: { classPropertyName: "$theme", publicName: "theme", isSignal: true, isRequired: false, transformFunction: null }, $onMonacoLoad: { classPropertyName: "$onMonacoLoad", publicName: "onMonacoLoad", isSignal: true, isRequired: false, transformFunction: null }, $disposeModelOnDestroy: { classPropertyName: "$disposeModelOnDestroy", publicName: "disposeModelOnDestroy", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { $value: "valueChange", editorLoaded: "editorLoaded" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => MonacoEditorComponent),
                multi: true,
            },
        ], ngImport: i0, template: '', isInline: true, styles: [":host{display:block}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MonacoEditorComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-monaco-editor', standalone: false, template: '', providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => MonacoEditorComponent),
                            multi: true,
                        },
                    ], changeDetection: ChangeDetectionStrategy.OnPush, styles: [":host{display:block}\n"] }]
        }] });

class TableauUiMonacoModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMonacoModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMonacoModule, declarations: [MonacoEditorComponent], imports: [CommonModule], exports: [MonacoEditorComponent] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMonacoModule, imports: [CommonModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiMonacoModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [MonacoEditorComponent],
                    exports: [MonacoEditorComponent],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { MonacoEditorComponent, TableauUiMonacoModule };
//# sourceMappingURL=tableau-ui-angular-monaco.mjs.map
