import * as i0 from '@angular/core';
import { contentChildren, effect, contentChild, viewChild, inject, ElementRef, model, input, computed, signal, Directive, Pipe, forwardRef, ChangeDetectionStrategy, Component, NgModule } from '@angular/core';
import { fromEvent, map } from 'rxjs';
import { OptionComponent, PrefixComponent, SuffixComponent, TableauUiCommonModule } from 'tableau-ui-angular/common';
import { DialogService, LocalStackOptions, TableauUiDialogModule } from 'tableau-ui-angular/dialog';
import { generateRandomString } from 'tableau-ui-angular/utils';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i2 from 'tableau-ui-angular/icon';
import { TableauUiIconModule } from 'tableau-ui-angular/icon';
import * as i3 from 'tableau-ui-angular/button';
import { TableauUiButtonModule } from 'tableau-ui-angular/button';

const SELECT_COMPONENT_HOST = {
    class: 'tab-input',
    '[attr.wrapping-mode]': '$selectedValueWrapMode()',
    '[tabindex]': '$disabled() ? -1 : 0',
    '(click)': 'openDropdown()',
    '(keydown)': 'onKeyDown($event)',
    '[attr.disabled]': '$disabled() ? true : null',
    '[aria-disabled]': '$disabled() ? true : null',
    '[aria-hidden]': '$disabled() ? true : null',
    '[id]': 'selectId',
};
class SelectBaseComponent {
    selectId;
    dropdownId;
    $options = contentChildren((OptionComponent));
    $isMultiSelect() {
        return false;
    }
    optionsChanged = effect(() => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const options = this.$options();
        // for (const op of options) {
        //     if (typeof op.$value() != typeof this.$selectedValue()) {
        //         console.warn(`ButtonToggleComponent: The type of an option '${typeof op.$value()}' (value: ${op.$value()}) does not match the expected type: '${typeof this.$selectedValue()}'. This may lead to unexpected behavior.`);
        //     }
        // }
        this.$highlightedOption.set(undefined);
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownPrefix = contentChild(PrefixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownSuffix = contentChild(SuffixComponent);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownTemplate = viewChild('dropdownTemplate');
    // #region Imports
    dialogService = inject(DialogService);
    elementRef = inject(ElementRef);
    // #endregion
    // #region Inputs
    /**
     * Whether the select is disabled
     * @remarks
     * Set this using the FormControl if you are using reactive forms.
     * Only set this manually if you are not using reactive forms.
     *
     * @default false
     */
    $disabled = model(false, {
        alias: 'disabled',
    });
    /**
     * Placeholder text to display when no value is selected
     * // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
     */
    $placeholder = input(undefined, {
        alias: 'placeholder',
    });
    /**
     * The currently selected value.
     * @remarks
     * If allowMultiple is true, this should be an array of values.
     * If allowMultiple is false, this should be a single value.
     */
    $value = model(undefined, {
        alias: 'value',
    });
    /**
     * The CSS text to apply to the dropdown container
     * @remarks
     * Use this to apply height, maxHeight, etc. to the dropdown container
     * @default '{}'
     */
    $dropdownContainerCss = input({}, {
        alias: 'dropdownContainerCss',
    });
    /**
     * The CSS text to apply to the options container in the dropdown
     * @remarks
     * Use this to apply height, maxHeight, etc. to the dropdown
     * This is applied to the options container, excluding the prefix & suffix
     *
     * @default "{ maxHeight: '300px', height: 'fit-content'}"
     */
    $dropdownOptionsContainerCss = input({
        maxHeight: '300px',
        height: 'fit-content',
    }, {
        alias: 'dropdownOptionsContainerCss',
    });
    /**
     * Whether to show the chevron icon on the right side of the select
     */
    $showChevron = input(true, {
        alias: 'showChevron',
    });
    /**
     * The location of the check icon in dropdown option if an option is selected
     * @default 'none'
     */
    $selectedCheckIconLocation = input('none', {
        alias: 'selectedCheckIconLocation',
    });
    /**
     * Highlight the selected item(s) with a primary back color
     * @default true
     */
    $selectedItemHighlight = input(true, {
        alias: 'selectedItemHighlight',
    });
    /**
     * Whether the clear button should be displayed
     * @remarks
     * It is only displayed if there are any values selected even if this is true
     * @default false
     */
    $allowClear = input(false, {
        alias: 'allowClear',
    });
    /**
     * The wrapping mode for the selected value field
     * @remarks
     * If set to 'wrap', the text will wrap to the next line if it is too long to fit on one line. This will increase the height of the control
     * If set to 'truncate', the text will be truncated with an ellipsis if it is too long to fit on one line. This will keep the height of the control the same
     * @default 'wrap'
     */
    $selectedValueWrapMode = input('wrap', {
        alias: 'selectedValueWrapMode',
    });
    /**
     * The template context to use for the selected value field
     * @remarks
     * Use this to display the 'icon' and 'text' properties of the selected value conditionally
     */
    $selectedValueTemplateContext = input({
        renderIcon: true,
        renderText: true,
    }, {
        alias: 'selectedValueTemplateContext',
    });
    /**
     * The template context to use for the dropdown options
     * @remarks
     * Use this to display the 'icon', 'text', and 'hint' properties of the options conditionally
     */
    $dropdownValueTemplateContext = input({
        renderIcon: true,
        renderText: true,
        renderHint: true,
    }, {
        alias: 'dropdownValueTemplateContext',
    });
    // #endregion
    // #region Constructor + Init + Destroy
    constructor() {
        const id = generateRandomString();
        this.selectId = `select-${id}`;
        this.dropdownId = `dropdown-${id}`;
    }
    ngAfterViewInit() {
        this.registerFocusChange();
    }
    ngOnDestroy() {
        this.unregisterFocusChange();
    }
    $selectedValueTemplates = computed(() => {
        if (!this.$hasValue()) {
            return [];
        }
        let val = [];
        const value = this.$value();
        if (Array.isArray(value)) {
            val = value;
        }
        else if (value !== undefined) {
            val = [value];
        }
        const ret = val.map((v) => ({
            value: v,
            template: this.$options()
                .find((o) => o.$value() === v)
                ?.$lineTemplate(),
        }));
        return ret.filter((e) => e.template !== undefined);
    });
    $displayClearButton = computed(() => {
        if (!this.$allowClear()) {
            return false;
        }
        if (!this.$hasValue()) {
            return false;
        }
        return true;
    });
    // #endregion
    // #region ControlValueAccessor
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    onChange = (value) => { };
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    onTouched = () => { };
    writeValue(value) {
        this.$value.set(value);
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.$disabled.set(isDisabled);
    }
    // #endregion
    // #region Value selection
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $highlightedOption = signal(undefined);
    optionMouseDown(event) {
        event.preventDefault();
        event.stopPropagation();
    }
    selectValue(option) {
        if (!this.$disabled() && !option.$disabled()) {
            const value = this.$value();
            const optionValue = option.$value();
            this.selectValueInternal(value, optionValue);
            this.onChange(this.$value());
            this.onTouched();
            if (!this.$isMultiSelect()) {
                this.$dropdownReference()?.close();
            }
        }
    }
    clearValue(e) {
        e.preventDefault();
        e.stopPropagation();
        if (this.$disabled()) {
            return;
        }
        // if (this.isMultiSelect) {
        //     this.$value.set([]);
        // } else {
        //     this.$value.set(undefined);
        // }
        this.clearValueInternal();
        this.onChange(this.$value());
        this.onTouched();
    }
    // isValueSelected(option: OptionComponent<TOption>) {
    //     const value = this.$value();
    //     const optionValue = option.$value();
    //     if (this.$allowMultiple()) {
    //         if (!Array.isArray(value)) {
    //             return false;
    //         }
    //         return value.includes(optionValue);
    //     } else {
    //         return value === optionValue;
    //     }
    // }
    // #endregion
    // #region Dropdown
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $dropdownReference = signal(undefined);
    $dropdownOpen = computed(() => this.$dropdownReference() !== undefined);
    openDropdown() {
        if (this.$disabled()) {
            return;
        }
        this.$highlightedOption.set(undefined);
        const elRect = this.elementRef.nativeElement.getBoundingClientRect();
        const ref = this.dialogService.openTemplateDialog(this.$dropdownTemplate(), {
            top(actualWidth, actualHeight) {
                if (elRect.bottom + actualHeight > window.innerHeight && elRect.top - actualHeight > 0) {
                    return elRect.top - actualHeight + 'px';
                }
                return elRect.bottom + 'px';
            },
            left: elRect.left + 'px',
            width: elRect.width + 'px',
            closeOnEscape: true,
            closeOnBackdropClick: true,
        }, null, new LocalStackOptions(this.elementRef.nativeElement));
        this.registerOptionKeyNavigation();
        ref.closed$.subscribe(() => {
            this.unregisterOptionKeyNavigation();
            if (this.$dropdownReference() === ref) {
                this.$dropdownReference.set(undefined);
            }
        });
        this.$dropdownReference.set(ref);
    }
    // #endregion
    // #region Keyboard navigation
    /**
     *
     * @param e Handles KeyDown event for:
     * - host element
     * - global keydown event when dropdown is open
     * @returns
     */
    onKeyDown(e) {
        if (e.key === 'Enter' || e.key === ' ') {
            const ref = this.$dropdownReference();
            // if dropdown is not open
            if (!ref) {
                this.openDropdown();
            }
            else {
                // if dropdown is open
                // if an option is highlighted, select it
                if (this.$highlightedOption()) {
                    this.selectValue(this.$highlightedOption());
                }
                else {
                    // if an option is not highlighted, close the dropdown
                    ref.close();
                }
            }
            e.preventDefault();
            e.stopPropagation();
        }
        if (e.key === 'ArrowUp' || e.key === 'ArrowDown') {
            // if dropdown is open:
            // - we find the currently HIGHLIGHTED option
            // - we find the next option to highlight
            // - if no option is found, we highlight the first/last option
            // if dropdown is NOT open:
            // - if this is a multiselect, ignore
            // - if this is a single select, find the next item to select
            // - if no item is selected, highlight the first/last item
            const open = this.$dropdownOpen();
            if (this.$isMultiSelect() && !open) {
                return;
            }
            let currentIndex = -1;
            if (open) {
                // find already highlighted option
                if (this.$highlightedOption()) {
                    currentIndex = this.$options().findIndex((o) => o.$value() === this.$highlightedOption().$value());
                }
            }
            else {
                // find already selected option
                currentIndex = this.$options().findIndex(
                // we can be sure that at this point, this is a single select dropdown, so this.value() can be safely used
                (o) => o.$value() === this.$value());
            }
            let nextIndex;
            // find the next index to highlight/select
            if (e.key === 'ArrowDown') {
                if (currentIndex === -1) {
                    // find the first non disabled option
                    nextIndex = this.$options().findIndex((o) => !o.$disabled());
                }
                else {
                    // find the next option that is not disabled
                    nextIndex = this.$options().findIndex((o, i) => i > currentIndex && !o.$disabled());
                    // if no option is found, find the next option that is not disabled before the current item
                    if (nextIndex === -1) {
                        nextIndex = this.$options().findIndex((o, i) => i < currentIndex && !o.$disabled());
                    }
                }
            }
            else if (e.key === 'ArrowUp') {
                if (currentIndex === -1) {
                    // find the last non disabled option
                    nextIndex = this.$options()
                        .slice()
                        .reverse()
                        .findIndex((o) => !o.$disabled());
                    if (nextIndex !== -1) {
                        nextIndex = this.$options().length - nextIndex - 1;
                    }
                }
                else {
                    const flippedCurrentIndex = this.$options().length - currentIndex - 1;
                    // find the next option that is not disabled
                    nextIndex = [...this.$options()].reverse().findIndex((o, i) => i > flippedCurrentIndex && !o.$disabled());
                    // if no option is found, find the next option that is not disabled before the current item
                    if (nextIndex === -1) {
                        nextIndex = [...this.$options()].reverse().findIndex((o, i) => i < flippedCurrentIndex && !o.$disabled());
                    }
                    if (nextIndex !== -1) {
                        nextIndex = this.$options().length - nextIndex - 1;
                    }
                }
            }
            else {
                nextIndex = -1;
            }
            if (nextIndex !== -1) {
                if (open) {
                    this.$highlightedOption.set(this.$options()[nextIndex]);
                    setTimeout(() => document.querySelector(`#${this.dropdownId} .option-wrapper.highlight`)?.scrollIntoView({
                        block: 'nearest',
                    }), 10);
                }
                else {
                    this.selectValue(this.$options()[nextIndex]);
                }
            }
            e.preventDefault();
            e.stopPropagation();
        }
    }
    clearKeyDown(e) {
        if (e.key === 'Enter' || e.key === ' ') {
            this.clearValue(e);
            e.preventDefault();
        }
    }
    optionKeyNavSubscription;
    registerOptionKeyNavigation() {
        this.unregisterOptionKeyNavigation();
        this.optionKeyNavSubscription = fromEvent(document, 'keydown')
            .pipe(map((e) => e))
            .subscribe((e) => {
            this.onKeyDown(e);
        });
    }
    unregisterOptionKeyNavigation() {
        if (this.optionKeyNavSubscription) {
            this.optionKeyNavSubscription.unsubscribe();
            this.optionKeyNavSubscription = undefined;
        }
    }
    // #endregion
    // #region Focus management
    focusChangeFn;
    registerFocusChange() {
        this.focusChangeFn = this.checkFocus.bind(this);
        document.addEventListener('focusout', this.focusChangeFn);
    }
    unregisterFocusChange() {
        document.removeEventListener('focusout', this.focusChangeFn);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    checkFocus(event) {
        setTimeout(() => {
            const dropdownContainer = document.getElementById(this.dropdownId);
            if (!this.elementRef.nativeElement.contains(document.activeElement) && dropdownContainer?.contains(document.activeElement) !== true) {
                const ref = this.$dropdownReference();
                if (ref) {
                    ref.close();
                }
            }
        }, 100);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: SelectBaseComponent, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.2.0", version: "20.0.3", type: SelectBaseComponent, isStandalone: true, inputs: { $disabled: { classPropertyName: "$disabled", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null }, $placeholder: { classPropertyName: "$placeholder", publicName: "placeholder", isSignal: true, isRequired: false, transformFunction: null }, $value: { classPropertyName: "$value", publicName: "value", isSignal: true, isRequired: false, transformFunction: null }, $dropdownContainerCss: { classPropertyName: "$dropdownContainerCss", publicName: "dropdownContainerCss", isSignal: true, isRequired: false, transformFunction: null }, $dropdownOptionsContainerCss: { classPropertyName: "$dropdownOptionsContainerCss", publicName: "dropdownOptionsContainerCss", isSignal: true, isRequired: false, transformFunction: null }, $showChevron: { classPropertyName: "$showChevron", publicName: "showChevron", isSignal: true, isRequired: false, transformFunction: null }, $selectedCheckIconLocation: { classPropertyName: "$selectedCheckIconLocation", publicName: "selectedCheckIconLocation", isSignal: true, isRequired: false, transformFunction: null }, $selectedItemHighlight: { classPropertyName: "$selectedItemHighlight", publicName: "selectedItemHighlight", isSignal: true, isRequired: false, transformFunction: null }, $allowClear: { classPropertyName: "$allowClear", publicName: "allowClear", isSignal: true, isRequired: false, transformFunction: null }, $selectedValueWrapMode: { classPropertyName: "$selectedValueWrapMode", publicName: "selectedValueWrapMode", isSignal: true, isRequired: false, transformFunction: null }, $selectedValueTemplateContext: { classPropertyName: "$selectedValueTemplateContext", publicName: "selectedValueTemplateContext", isSignal: true, isRequired: false, transformFunction: null }, $dropdownValueTemplateContext: { classPropertyName: "$dropdownValueTemplateContext", publicName: "dropdownValueTemplateContext", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { $disabled: "disabledChange", $value: "valueChange" }, queries: [{ propertyName: "$options", predicate: (OptionComponent), isSignal: true }, { propertyName: "$dropdownPrefix", first: true, predicate: PrefixComponent, descendants: true, isSignal: true }, { propertyName: "$dropdownSuffix", first: true, predicate: SuffixComponent, descendants: true, isSignal: true }], viewQueries: [{ propertyName: "$dropdownTemplate", first: true, predicate: ["dropdownTemplate"], descendants: true, isSignal: true }], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: SelectBaseComponent, decorators: [{
            type: Directive
        }], ctorParameters: () => [] });

class IsSelectedValuePipe {
    transform(option, allowMultiple, value) {
        if (allowMultiple) {
            return value instanceof Array && value.includes(option.$value());
        }
        return option.$value() === value;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: IsSelectedValuePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: IsSelectedValuePipe, isStandalone: false, name: "isSelectedValue" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: IsSelectedValuePipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'isSelectedValue',
                    standalone: false,
                }]
        }] });

class MultipleTemplatePipe {
    transform(template, number) {
        return template.replace('{number}', number.toString());
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MultipleTemplatePipe, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: MultipleTemplatePipe, isStandalone: false, name: "multipleTemplate" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MultipleTemplatePipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'multipleTemplate',
                    standalone: false,
                }]
        }] });

class SingleSelectComponent extends SelectBaseComponent {
    $multipleSelectionMaxItemsListed = input(2);
    $hasValue = computed(() => {
        const value = this.$value();
        return value !== undefined;
    });
    selectValueInternal = (currentValue, selectedValue) => {
        this.$value.set(selectedValue);
    };
    clearValueInternal = () => {
        this.$value.set(undefined);
    };
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: SingleSelectComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: SingleSelectComponent, isStandalone: false, selector: "tab-single-select", inputs: { $multipleSelectionMaxItemsListed: { classPropertyName: "$multipleSelectionMaxItemsListed", publicName: "$multipleSelectionMaxItemsListed", isSignal: true, isRequired: false, transformFunction: null } }, host: { listeners: { "click": "openDropdown()", "keydown": "onKeyDown($event)" }, properties: { "attr.wrapping-mode": "$selectedValueWrapMode()", "tabindex": "$disabled() ? -1 : 0", "attr.disabled": "$disabled() ? true : null", "aria-disabled": "$disabled() ? true : null", "aria-hidden": "$disabled() ? true : null", "id": "selectId" }, classAttribute: "tab-input" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => SingleSelectComponent),
                multi: true,
            },
        ], usesInheritance: true, ngImport: i0, template: "<div class=\"content-display\">\n    @if ($selectedValueTemplates().length === 0) {\n        <span class=\"placeholder\">{{ $placeholder() }}</span>\n    } @else {\n        <span class=\"value\">\n            @if ($isMultiSelect() && $selectedValueTemplates().length > $multipleSelectionMaxItemsListed()) {\n                <ng-container>{{ $multipleSelectionNumberSelectedTemplate() | multipleTemplate: $selectedValueTemplates().length }}</ng-container>\n            } @else {\n                @for (v of $selectedValueTemplates(); track v.value; let last = $last) {\n                    <ng-container [ngTemplateOutlet]=\"v.template\" [ngTemplateOutletContext]=\"$selectedValueTemplateContext()\" />\n                    @if (!last) {\n                        <span>,</span>\n                    }\n                }\n            }\n        </span>\n    }\n</div>\n@if ($displayClearButton()) {\n    <button layout=\"icon\" color=\"plain\" tabindex=\"0\" [disabled]=\"$disabled()\" (click)=\"clearValue($event)\" (keydown)=\"clearKeyDown($event)\">\n        <tab-icon value=\"clear\" />\n    </button>\n}\n@if ($showChevron()) {\n    <tab-icon class=\"dropdown\" icon-button [value]=\"$dropdownOpen() ? 'keyboard_arrow_up' : 'keyboard_arrow_down'\" />\n}\n\n<ng-template #dropdownTemplate>\n    <div class=\"dropdown-container\" [id]=\"dropdownId\" [ngStyle]=\"$dropdownContainerCss()\">\n        @if ($dropdownPrefix()) {\n            <div class=\"options-prefix\">\n                <ng-content select=\"tab-prefix\" />\n            </div>\n        }\n        <div class=\"option-values\" [ngStyle]=\"$dropdownOptionsContainerCss()\">\n            @for (option of $options(); track option.$value()) {\n                <!-- eslint-disable-next-line @angular-eslint/template/interactive-supports-focus, @angular-eslint/template/click-events-have-key-events -->\n                <div\n                    class=\"option-wrapper\"\n                    [class.disabled]=\"option.$disabled()\"\n                    [class.highlight]=\"!!$highlightedOption() && $highlightedOption()!.$value() === option.$value()\"\n                    [class.selected]=\"(option | isSelectedValue: $isMultiSelect : $value()) && $selectedItemHighlight()\"\n                    (click)=\"selectValue(option)\"\n                    (mousedown)=\"optionMouseDown($event)\"\n                    (mouseenter)=\"$highlightedOption.set(option)\"\n                >\n                    <div class=\"check\" [ngClass]=\"$selectedCheckIconLocation()\">\n                        @if (option | isSelectedValue: $isMultiSelect : $value()) {\n                            <tab-icon value=\"check\" />\n                        }\n                    </div>\n                    <div class=\"option-content\">\n                        <ng-container [ngTemplateOutlet]=\"option.$template()\" [ngTemplateOutletContext]=\"$dropdownValueTemplateContext()\" />\n                    </div>\n                </div>\n            }\n        </div>\n        @if ($dropdownSuffix()) {\n            <div class=\"options-suffix\">\n                <ng-content select=\"tab-suffix\" />\n            </div>\n        }\n    </div>\n</ng-template>\n", styles: [":host{display:grid;align-items:center;grid-template-columns:1fr auto auto;-webkit-user-select:none;user-select:none}:host:focus .dropdown{color:var(--twc-color-primary)}:host[disabled]{pointer-events:none}.content-display{padding-top:.25em;padding-bottom:.25em;line-height:1}:host[wrapping-mode=truncate] .content-display{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.dropdown{font-size:1.5em}.dropdown-container{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow:hidden auto;-webkit-user-select:none;user-select:none}.dropdown-container.open{display:block}.dropdown-container .option-values{overflow:hidden auto;padding:.5rem}.dropdown-container .option-values .option-wrapper{display:grid;grid-template-columns:auto 1fr auto;align-items:center;border-radius:2px;padding-left:.5em;padding-right:.5em;border:1px solid transparent;margin-top:-1px}.dropdown-container .option-values .option-wrapper .check{font-size:1.2em;width:1.2em;line-height:1;grid-row:1}.dropdown-container .option-values .option-wrapper .check.left{grid-column:1;margin-right:.25em}.dropdown-container .option-values .option-wrapper .check.right{grid-column:3;margin-left:.25em}.dropdown-container .option-values .option-wrapper .check.none{display:none}.dropdown-container .option-values .option-wrapper .option-content{grid-row:1;grid-column:2;padding:.25em 0}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight{background-color:var(--twc-color-highlight);cursor:pointer}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight:not(.selected){outline:var(--twc-menu-highlight-outline);outline-offset:var(--twc-menu-highlight-outline-offset)}.dropdown-container .option-values .option-wrapper.selected{background-color:var(--twc-menu-selected-background-color);outline:var(--twc-menu-selected-outline);outline-offset:var(--twc-menu-selected-outline-offset)}.dropdown-container .option-values .option-wrapper ::ng-deep .tab-option{line-height:normal}\n"], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { kind: "component", type: i2.IconComponent, selector: "tab-icon", inputs: ["value", "color", "type", "fill", "weight", "grade", "opticalSizePx"] }, { kind: "component", type: i3.ButtonComponent, selector: "button:not([tab-menu-button]),a[button]:not([tab-menu-button])", inputs: ["disabled", "loading", "tabindex", "type", "kind", "color", "layout", "inline", "noFocusOnClick"] }, { kind: "pipe", type: IsSelectedValuePipe, name: "isSelectedValue" }, { kind: "pipe", type: MultipleTemplatePipe, name: "multipleTemplate" }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: SingleSelectComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-single-select', standalone: false, providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => SingleSelectComponent),
                            multi: true,
                        },
                    ], changeDetection: ChangeDetectionStrategy.OnPush, host: SELECT_COMPONENT_HOST, template: "<div class=\"content-display\">\n    @if ($selectedValueTemplates().length === 0) {\n        <span class=\"placeholder\">{{ $placeholder() }}</span>\n    } @else {\n        <span class=\"value\">\n            @if ($isMultiSelect() && $selectedValueTemplates().length > $multipleSelectionMaxItemsListed()) {\n                <ng-container>{{ $multipleSelectionNumberSelectedTemplate() | multipleTemplate: $selectedValueTemplates().length }}</ng-container>\n            } @else {\n                @for (v of $selectedValueTemplates(); track v.value; let last = $last) {\n                    <ng-container [ngTemplateOutlet]=\"v.template\" [ngTemplateOutletContext]=\"$selectedValueTemplateContext()\" />\n                    @if (!last) {\n                        <span>,</span>\n                    }\n                }\n            }\n        </span>\n    }\n</div>\n@if ($displayClearButton()) {\n    <button layout=\"icon\" color=\"plain\" tabindex=\"0\" [disabled]=\"$disabled()\" (click)=\"clearValue($event)\" (keydown)=\"clearKeyDown($event)\">\n        <tab-icon value=\"clear\" />\n    </button>\n}\n@if ($showChevron()) {\n    <tab-icon class=\"dropdown\" icon-button [value]=\"$dropdownOpen() ? 'keyboard_arrow_up' : 'keyboard_arrow_down'\" />\n}\n\n<ng-template #dropdownTemplate>\n    <div class=\"dropdown-container\" [id]=\"dropdownId\" [ngStyle]=\"$dropdownContainerCss()\">\n        @if ($dropdownPrefix()) {\n            <div class=\"options-prefix\">\n                <ng-content select=\"tab-prefix\" />\n            </div>\n        }\n        <div class=\"option-values\" [ngStyle]=\"$dropdownOptionsContainerCss()\">\n            @for (option of $options(); track option.$value()) {\n                <!-- eslint-disable-next-line @angular-eslint/template/interactive-supports-focus, @angular-eslint/template/click-events-have-key-events -->\n                <div\n                    class=\"option-wrapper\"\n                    [class.disabled]=\"option.$disabled()\"\n                    [class.highlight]=\"!!$highlightedOption() && $highlightedOption()!.$value() === option.$value()\"\n                    [class.selected]=\"(option | isSelectedValue: $isMultiSelect : $value()) && $selectedItemHighlight()\"\n                    (click)=\"selectValue(option)\"\n                    (mousedown)=\"optionMouseDown($event)\"\n                    (mouseenter)=\"$highlightedOption.set(option)\"\n                >\n                    <div class=\"check\" [ngClass]=\"$selectedCheckIconLocation()\">\n                        @if (option | isSelectedValue: $isMultiSelect : $value()) {\n                            <tab-icon value=\"check\" />\n                        }\n                    </div>\n                    <div class=\"option-content\">\n                        <ng-container [ngTemplateOutlet]=\"option.$template()\" [ngTemplateOutletContext]=\"$dropdownValueTemplateContext()\" />\n                    </div>\n                </div>\n            }\n        </div>\n        @if ($dropdownSuffix()) {\n            <div class=\"options-suffix\">\n                <ng-content select=\"tab-suffix\" />\n            </div>\n        }\n    </div>\n</ng-template>\n", styles: [":host{display:grid;align-items:center;grid-template-columns:1fr auto auto;-webkit-user-select:none;user-select:none}:host:focus .dropdown{color:var(--twc-color-primary)}:host[disabled]{pointer-events:none}.content-display{padding-top:.25em;padding-bottom:.25em;line-height:1}:host[wrapping-mode=truncate] .content-display{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.dropdown{font-size:1.5em}.dropdown-container{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow:hidden auto;-webkit-user-select:none;user-select:none}.dropdown-container.open{display:block}.dropdown-container .option-values{overflow:hidden auto;padding:.5rem}.dropdown-container .option-values .option-wrapper{display:grid;grid-template-columns:auto 1fr auto;align-items:center;border-radius:2px;padding-left:.5em;padding-right:.5em;border:1px solid transparent;margin-top:-1px}.dropdown-container .option-values .option-wrapper .check{font-size:1.2em;width:1.2em;line-height:1;grid-row:1}.dropdown-container .option-values .option-wrapper .check.left{grid-column:1;margin-right:.25em}.dropdown-container .option-values .option-wrapper .check.right{grid-column:3;margin-left:.25em}.dropdown-container .option-values .option-wrapper .check.none{display:none}.dropdown-container .option-values .option-wrapper .option-content{grid-row:1;grid-column:2;padding:.25em 0}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight{background-color:var(--twc-color-highlight);cursor:pointer}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight:not(.selected){outline:var(--twc-menu-highlight-outline);outline-offset:var(--twc-menu-highlight-outline-offset)}.dropdown-container .option-values .option-wrapper.selected{background-color:var(--twc-menu-selected-background-color);outline:var(--twc-menu-selected-outline);outline-offset:var(--twc-menu-selected-outline-offset)}.dropdown-container .option-values .option-wrapper ::ng-deep .tab-option{line-height:normal}\n"] }]
        }] });

class MultiSelectComponent extends SelectBaseComponent {
    $isMultiSelect() {
        return true;
    }
    /**
     * The maximum number of items to display in the selected value field when multiple items are selected
     * @default 2
     */
    $multipleSelectionMaxItemsListed = input(2, {
        alias: 'multipleSelectionMaxItemsListed',
    });
    /**
     * The display template to use when more than the maximum number of items selected
     * @remarks
     * Use {number} as a placeholder for the number of items selected
     */
    $multipleSelectionNumberSelectedTemplate = input('{number} items selected', {
        alias: 'multipleSelectionNumberSelectedTemplate',
    });
    $hasValue = computed(() => {
        const value = this.$value();
        if (value === undefined) {
            return false;
        }
        if (!Array.isArray(value)) {
            return false;
        }
        return value.length > 0;
    });
    selectValueInternal = (currentValue, selectedValue) => {
        if (currentValue === undefined || !Array.isArray(currentValue)) {
            this.$value.set([selectedValue]);
        }
        else if (!currentValue.includes(selectedValue)) {
            this.$value.set([...currentValue, selectedValue]);
        }
        else {
            this.$value.set(currentValue.filter((e) => e !== selectedValue));
        }
    };
    clearValueInternal = () => {
        this.$value.set([]);
    };
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MultiSelectComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: MultiSelectComponent, isStandalone: false, selector: "tab-multi-select", inputs: { $multipleSelectionMaxItemsListed: { classPropertyName: "$multipleSelectionMaxItemsListed", publicName: "multipleSelectionMaxItemsListed", isSignal: true, isRequired: false, transformFunction: null }, $multipleSelectionNumberSelectedTemplate: { classPropertyName: "$multipleSelectionNumberSelectedTemplate", publicName: "multipleSelectionNumberSelectedTemplate", isSignal: true, isRequired: false, transformFunction: null } }, host: { listeners: { "click": "openDropdown()", "keydown": "onKeyDown($event)" }, properties: { "attr.wrapping-mode": "$selectedValueWrapMode()", "tabindex": "$disabled() ? -1 : 0", "attr.disabled": "$disabled() ? true : null", "aria-disabled": "$disabled() ? true : null", "aria-hidden": "$disabled() ? true : null", "id": "selectId" }, classAttribute: "tab-input" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => MultiSelectComponent),
                multi: true,
            },
        ], usesInheritance: true, ngImport: i0, template: "<div class=\"content-display\">\n    @if ($selectedValueTemplates().length === 0) {\n        <span class=\"placeholder\">{{ $placeholder() }}</span>\n    } @else {\n        <span class=\"value\">\n            @if ($isMultiSelect() && $selectedValueTemplates().length > $multipleSelectionMaxItemsListed()) {\n                <ng-container>{{ $multipleSelectionNumberSelectedTemplate() | multipleTemplate: $selectedValueTemplates().length }}</ng-container>\n            } @else {\n                @for (v of $selectedValueTemplates(); track v.value; let last = $last) {\n                    <ng-container [ngTemplateOutlet]=\"v.template\" [ngTemplateOutletContext]=\"$selectedValueTemplateContext()\" />\n                    @if (!last) {\n                        <span>,</span>\n                    }\n                }\n            }\n        </span>\n    }\n</div>\n@if ($displayClearButton()) {\n    <button layout=\"icon\" color=\"plain\" tabindex=\"0\" [disabled]=\"$disabled()\" (click)=\"clearValue($event)\" (keydown)=\"clearKeyDown($event)\">\n        <tab-icon value=\"clear\" />\n    </button>\n}\n@if ($showChevron()) {\n    <tab-icon class=\"dropdown\" icon-button [value]=\"$dropdownOpen() ? 'keyboard_arrow_up' : 'keyboard_arrow_down'\" />\n}\n\n<ng-template #dropdownTemplate>\n    <div class=\"dropdown-container\" [id]=\"dropdownId\" [ngStyle]=\"$dropdownContainerCss()\">\n        @if ($dropdownPrefix()) {\n            <div class=\"options-prefix\">\n                <ng-content select=\"tab-prefix\" />\n            </div>\n        }\n        <div class=\"option-values\" [ngStyle]=\"$dropdownOptionsContainerCss()\">\n            @for (option of $options(); track option.$value()) {\n                <!-- eslint-disable-next-line @angular-eslint/template/interactive-supports-focus, @angular-eslint/template/click-events-have-key-events -->\n                <div\n                    class=\"option-wrapper\"\n                    [class.disabled]=\"option.$disabled()\"\n                    [class.highlight]=\"!!$highlightedOption() && $highlightedOption()!.$value() === option.$value()\"\n                    [class.selected]=\"(option | isSelectedValue: $isMultiSelect : $value()) && $selectedItemHighlight()\"\n                    (click)=\"selectValue(option)\"\n                    (mousedown)=\"optionMouseDown($event)\"\n                    (mouseenter)=\"$highlightedOption.set(option)\"\n                >\n                    <div class=\"check\" [ngClass]=\"$selectedCheckIconLocation()\">\n                        @if (option | isSelectedValue: $isMultiSelect : $value()) {\n                            <tab-icon value=\"check\" />\n                        }\n                    </div>\n                    <div class=\"option-content\">\n                        <ng-container [ngTemplateOutlet]=\"option.$template()\" [ngTemplateOutletContext]=\"$dropdownValueTemplateContext()\" />\n                    </div>\n                </div>\n            }\n        </div>\n        @if ($dropdownSuffix()) {\n            <div class=\"options-suffix\">\n                <ng-content select=\"tab-suffix\" />\n            </div>\n        }\n    </div>\n</ng-template>\n", styles: [":host{display:grid;align-items:center;grid-template-columns:1fr auto auto;-webkit-user-select:none;user-select:none}:host:focus .dropdown{color:var(--twc-color-primary)}:host[disabled]{pointer-events:none}.content-display{padding-top:.25em;padding-bottom:.25em;line-height:1}:host[wrapping-mode=truncate] .content-display{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.dropdown{font-size:1.5em}.dropdown-container{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow:hidden auto;-webkit-user-select:none;user-select:none}.dropdown-container.open{display:block}.dropdown-container .option-values{overflow:hidden auto;padding:.5rem}.dropdown-container .option-values .option-wrapper{display:grid;grid-template-columns:auto 1fr auto;align-items:center;border-radius:2px;padding-left:.5em;padding-right:.5em;border:1px solid transparent;margin-top:-1px}.dropdown-container .option-values .option-wrapper .check{font-size:1.2em;width:1.2em;line-height:1;grid-row:1}.dropdown-container .option-values .option-wrapper .check.left{grid-column:1;margin-right:.25em}.dropdown-container .option-values .option-wrapper .check.right{grid-column:3;margin-left:.25em}.dropdown-container .option-values .option-wrapper .check.none{display:none}.dropdown-container .option-values .option-wrapper .option-content{grid-row:1;grid-column:2;padding:.25em 0}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight{background-color:var(--twc-color-highlight);cursor:pointer}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight:not(.selected){outline:var(--twc-menu-highlight-outline);outline-offset:var(--twc-menu-highlight-outline-offset)}.dropdown-container .option-values .option-wrapper.selected{background-color:var(--twc-menu-selected-background-color);outline:var(--twc-menu-selected-outline);outline-offset:var(--twc-menu-selected-outline-offset)}.dropdown-container .option-values .option-wrapper ::ng-deep .tab-option{line-height:normal}\n"], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { kind: "component", type: i2.IconComponent, selector: "tab-icon", inputs: ["value", "color", "type", "fill", "weight", "grade", "opticalSizePx"] }, { kind: "component", type: i3.ButtonComponent, selector: "button:not([tab-menu-button]),a[button]:not([tab-menu-button])", inputs: ["disabled", "loading", "tabindex", "type", "kind", "color", "layout", "inline", "noFocusOnClick"] }, { kind: "pipe", type: IsSelectedValuePipe, name: "isSelectedValue" }, { kind: "pipe", type: MultipleTemplatePipe, name: "multipleTemplate" }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: MultiSelectComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-multi-select', standalone: false, providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => MultiSelectComponent),
                            multi: true,
                        },
                    ], changeDetection: ChangeDetectionStrategy.OnPush, host: SELECT_COMPONENT_HOST, template: "<div class=\"content-display\">\n    @if ($selectedValueTemplates().length === 0) {\n        <span class=\"placeholder\">{{ $placeholder() }}</span>\n    } @else {\n        <span class=\"value\">\n            @if ($isMultiSelect() && $selectedValueTemplates().length > $multipleSelectionMaxItemsListed()) {\n                <ng-container>{{ $multipleSelectionNumberSelectedTemplate() | multipleTemplate: $selectedValueTemplates().length }}</ng-container>\n            } @else {\n                @for (v of $selectedValueTemplates(); track v.value; let last = $last) {\n                    <ng-container [ngTemplateOutlet]=\"v.template\" [ngTemplateOutletContext]=\"$selectedValueTemplateContext()\" />\n                    @if (!last) {\n                        <span>,</span>\n                    }\n                }\n            }\n        </span>\n    }\n</div>\n@if ($displayClearButton()) {\n    <button layout=\"icon\" color=\"plain\" tabindex=\"0\" [disabled]=\"$disabled()\" (click)=\"clearValue($event)\" (keydown)=\"clearKeyDown($event)\">\n        <tab-icon value=\"clear\" />\n    </button>\n}\n@if ($showChevron()) {\n    <tab-icon class=\"dropdown\" icon-button [value]=\"$dropdownOpen() ? 'keyboard_arrow_up' : 'keyboard_arrow_down'\" />\n}\n\n<ng-template #dropdownTemplate>\n    <div class=\"dropdown-container\" [id]=\"dropdownId\" [ngStyle]=\"$dropdownContainerCss()\">\n        @if ($dropdownPrefix()) {\n            <div class=\"options-prefix\">\n                <ng-content select=\"tab-prefix\" />\n            </div>\n        }\n        <div class=\"option-values\" [ngStyle]=\"$dropdownOptionsContainerCss()\">\n            @for (option of $options(); track option.$value()) {\n                <!-- eslint-disable-next-line @angular-eslint/template/interactive-supports-focus, @angular-eslint/template/click-events-have-key-events -->\n                <div\n                    class=\"option-wrapper\"\n                    [class.disabled]=\"option.$disabled()\"\n                    [class.highlight]=\"!!$highlightedOption() && $highlightedOption()!.$value() === option.$value()\"\n                    [class.selected]=\"(option | isSelectedValue: $isMultiSelect : $value()) && $selectedItemHighlight()\"\n                    (click)=\"selectValue(option)\"\n                    (mousedown)=\"optionMouseDown($event)\"\n                    (mouseenter)=\"$highlightedOption.set(option)\"\n                >\n                    <div class=\"check\" [ngClass]=\"$selectedCheckIconLocation()\">\n                        @if (option | isSelectedValue: $isMultiSelect : $value()) {\n                            <tab-icon value=\"check\" />\n                        }\n                    </div>\n                    <div class=\"option-content\">\n                        <ng-container [ngTemplateOutlet]=\"option.$template()\" [ngTemplateOutletContext]=\"$dropdownValueTemplateContext()\" />\n                    </div>\n                </div>\n            }\n        </div>\n        @if ($dropdownSuffix()) {\n            <div class=\"options-suffix\">\n                <ng-content select=\"tab-suffix\" />\n            </div>\n        }\n    </div>\n</ng-template>\n", styles: [":host{display:grid;align-items:center;grid-template-columns:1fr auto auto;-webkit-user-select:none;user-select:none}:host:focus .dropdown{color:var(--twc-color-primary)}:host[disabled]{pointer-events:none}.content-display{padding-top:.25em;padding-bottom:.25em;line-height:1}:host[wrapping-mode=truncate] .content-display{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.dropdown{font-size:1.5em}.dropdown-container{background:var(--twc-color-base);border-color:var(--twc-color-border-light);border-radius:var(--twc-menu-border-radius);border-style:solid;border-width:1px;box-shadow:var(--twc-dialog-box-shadow);outline:none;color:var(--twc-color-text);line-height:normal;overflow:hidden auto;-webkit-user-select:none;user-select:none}.dropdown-container.open{display:block}.dropdown-container .option-values{overflow:hidden auto;padding:.5rem}.dropdown-container .option-values .option-wrapper{display:grid;grid-template-columns:auto 1fr auto;align-items:center;border-radius:2px;padding-left:.5em;padding-right:.5em;border:1px solid transparent;margin-top:-1px}.dropdown-container .option-values .option-wrapper .check{font-size:1.2em;width:1.2em;line-height:1;grid-row:1}.dropdown-container .option-values .option-wrapper .check.left{grid-column:1;margin-right:.25em}.dropdown-container .option-values .option-wrapper .check.right{grid-column:3;margin-left:.25em}.dropdown-container .option-values .option-wrapper .check.none{display:none}.dropdown-container .option-values .option-wrapper .option-content{grid-row:1;grid-column:2;padding:.25em 0}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight{background-color:var(--twc-color-highlight);cursor:pointer}.dropdown-container .option-values .option-wrapper:not(.disabled).highlight:not(.selected){outline:var(--twc-menu-highlight-outline);outline-offset:var(--twc-menu-highlight-outline-offset)}.dropdown-container .option-values .option-wrapper.selected{background-color:var(--twc-menu-selected-background-color);outline:var(--twc-menu-selected-outline);outline-offset:var(--twc-menu-selected-outline-offset)}.dropdown-container .option-values .option-wrapper ::ng-deep .tab-option{line-height:normal}\n"] }]
        }] });

class TableauUiSelectModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiSelectModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiSelectModule, declarations: [IsSelectedValuePipe, MultipleTemplatePipe, SingleSelectComponent, MultiSelectComponent], imports: [CommonModule, TableauUiCommonModule, TableauUiIconModule, TableauUiButtonModule, TableauUiDialogModule], exports: [SingleSelectComponent, MultiSelectComponent] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiSelectModule, imports: [CommonModule, TableauUiCommonModule, TableauUiIconModule, TableauUiButtonModule, TableauUiDialogModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiSelectModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, TableauUiCommonModule, TableauUiIconModule, TableauUiButtonModule, TableauUiDialogModule],
                    declarations: [IsSelectedValuePipe, MultipleTemplatePipe, SingleSelectComponent, MultiSelectComponent],
                    exports: [SingleSelectComponent, MultiSelectComponent],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { MultiSelectComponent, SingleSelectComponent, TableauUiSelectModule };
//# sourceMappingURL=tableau-ui-angular-select.mjs.map
