import * as i0 from '@angular/core';
import { signal, inject, ElementRef, input, effect, computed, ChangeDetectionStrategy, Component, Directive, model, output, contentChild, viewChild, forwardRef, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { toObservable } from '@angular/core/rxjs-interop';
import { generateRandomString } from 'tableau-ui-angular/utils';
import * as i1$1 from 'tableau-ui-angular/button';
import { TableauUiButtonModule } from 'tableau-ui-angular/button';
import * as i2 from 'tableau-ui-angular/icon';
import { TableauUiIconModule } from 'tableau-ui-angular/icon';
import * as i3 from 'tableau-ui-angular/tooltip';
import { TableauUiTooltipModule } from 'tableau-ui-angular/tooltip';

class TreeNodeRegistry {
    nodes = signal([]);
    register(node) {
        const nodes = this.nodes();
        const existingNode = nodes.find((n) => n.$hierarchyId() === node.$hierarchyId());
        if (!existingNode) {
            this.nodes.set([...nodes, node]);
        }
    }
    unregister(node) {
        const nodes = this.nodes();
        const filteredNodes = nodes.filter((n) => n.$hierarchyId() !== node.$hierarchyId());
        if (filteredNodes.length !== nodes.length) {
            this.nodes.set(filteredNodes);
        }
    }
}

class TabTreeComponent {
    selfElementRef = inject(ElementRef);
    $showRootGridLines = input(false, {
        alias: 'showRootGridLines',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $gridLinesBorder = input(undefined, {
        alias: 'gridLinesBorder',
    });
    $gridLinesBorderRadius = input('0px', {
        alias: 'gridLinesBorderRadius',
    });
    $expandButtonSize = input('1.2rem', {
        alias: 'expandButtonSize',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $expandButtonColor = input(undefined, {
        alias: 'expandButtonColor',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $expandButtonGap = input('0.2rem', {
        alias: 'expandButtonGap',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $expandButtonTooltip = input(undefined, {
        alias: 'expandButtonTooltip',
    });
    $expandButtonAlign = input('center', {
        alias: 'expandButtonAlign',
    });
    $childrenIndent = input('1.4rem', {
        alias: 'childrenIndent',
    });
    $keepButtonOffsetOnNoChildren = input(true, {
        alias: 'keepButtonOffsetOnNoChildren',
    });
    $hierarchyMode = input('auto', {
        alias: 'hierarchyMode',
    });
    registry = new TreeNodeRegistry();
    // used when hierarchyMode is auto to get direct children
    // directChildren = contentChildren(TabTreeNodeComponent);
    templateParams = {
        $childrenIndent: this.$childrenIndent,
        $expandButtonSize: this.$expandButtonSize,
        $expandButtonColor: this.$expandButtonColor,
        $expandButtonGap: this.$expandButtonGap,
        $expandButtonTooltip: this.$expandButtonTooltip,
        $expandButtonAlign: this.$expandButtonAlign,
        $keepButtonOffsetOnNoChildren: this.$keepButtonOffsetOnNoChildren,
    };
    constructor() {
        effect(() => {
            // set the hierarchyMode of each children all times
            const registryNodes = this.registry.nodes();
            for (const child of registryNodes.filter((e) => e.$hierarchyParentId() === undefined)) {
                if (this.$hierarchyMode() === 'auto') {
                    this.hierarchyModeAutoSetChildrenDepthAndId(registryNodes, child, null, this.$showRootGridLines() ? 1 : 0);
                }
                else {
                    this.hierarchyModeManualSetChildrenDepthAndId(child, this.$showRootGridLines() ? 1 : 0);
                }
            }
            if (this.$gridLinesBorder() !== undefined) {
                this.redrawGridLines();
            }
        });
    }
    hierarchyModeAutoSetChildrenDepthAndId(allNodes, node, parentNode = null, depth) {
        node.$depth.set(depth);
        node.$parent.set(parentNode);
        node.$children.set(allNodes.filter((e) => e.$hierarchyParentId() === node.$hierarchyId()));
        for (const c of node.$children()) {
            this.hierarchyModeAutoSetChildrenDepthAndId(allNodes, c, node, depth + 1);
        }
    }
    hierarchyModeManualSetChildrenDepthAndId(child, currentChildDepth) {
        child.$depth.set(currentChildDepth);
        // get direct children
        const children = this.registry.nodes().filter((e) => e.$hierarchyParentId() === child.$hierarchyId());
        child.$children.set(children);
        for (const c of children) {
            c.$parent.set(child);
            this.hierarchyModeManualSetChildrenDepthAndId(c, currentChildDepth + 1);
        }
    }
    $childrenDisplayOrder = computed(() => {
        const rootChildren = this.registry
            .nodes()
            .filter((e) => e.$parent() === null)
            .sort((a, b) => (a.$order() > b.$order() ? 1 : a.$order() < b.$order() ? -1 : 0));
        const ret = [];
        for (const child of rootChildren) {
            ret.push(...this.getChildrenDisplayOrder(child));
        }
        return ret;
    });
    getChildrenDisplayOrder(node) {
        const ret = [node];
        const sortedChildren = node.$children().sort((a, b) => (a.$order() > b.$order() ? 1 : a.$order() < b.$order() ? -1 : 0));
        for (const child of sortedChildren) {
            ret.push(...this.getChildrenDisplayOrder(child));
        }
        return ret;
    }
    resizeObserver;
    ngAfterContentInit() {
        this.resizeObserver = new ResizeObserver(() => {
            this.redrawGridLines();
        });
        this.resizeObserver.observe(this.selfElementRef.nativeElement);
    }
    ngOnDestroy() {
        this.resizeObserver?.disconnect();
    }
    existingGridLines = [];
    redrawCounter = 0;
    redrawGridLines() {
        if (this.$gridLinesBorder() === undefined) {
            return;
        }
        // if the redrawCounter is not 0, we are already redrawing
        const currentRedrawCounter = this.redrawCounter + 1;
        this.redrawCounter = currentRedrawCounter;
        const gridLines = [];
        for (const child of this.registry.nodes()) {
            if (this.redrawCounter !== currentRedrawCounter) {
                return;
            }
            const headerButtonRect = document.getElementById(child.headerButtonId)?.getBoundingClientRect();
            // root gridlines
            if (this.$showRootGridLines() && !child.$parent()) {
                const selfRect = this.selfElementRef.nativeElement.getBoundingClientRect();
                if (!headerButtonRect) {
                    continue;
                }
                if (this.redrawCounter !== currentRedrawCounter) {
                    return;
                }
                gridLines.push({
                    id: child.$hierarchyId(),
                    from: {
                        top: 0,
                        left: 0,
                    },
                    to: {
                        top: headerButtonRect.top - selfRect.top + headerButtonRect.height / 2,
                        left: headerButtonRect.x - selfRect.x + (child.$children().length === 0 ? headerButtonRect.width : 0),
                    },
                });
                continue;
            }
            if (this.redrawCounter !== currentRedrawCounter) {
                return;
            }
            // show children gridlines
            if (child.$depth() === 0 || !child.$parent() || !child.$parent().$expanded() || headerButtonRect === undefined) {
                continue;
            }
            const selfRect = this.selfElementRef.nativeElement.getBoundingClientRect();
            const parentButtonRect = document.getElementById(child.$parent().headerButtonId)?.getBoundingClientRect();
            if (!parentButtonRect) {
                continue;
            }
            if (this.redrawCounter !== currentRedrawCounter) {
                return;
            }
            gridLines.push({
                id: child.$hierarchyId(),
                from: {
                    top: parentButtonRect.top - selfRect.top + parentButtonRect.height,
                    left: parentButtonRect.x - selfRect.left + parentButtonRect.width / 2,
                },
                to: {
                    top: headerButtonRect.top - selfRect.top + headerButtonRect.height / 2,
                    left: headerButtonRect.x - selfRect.x + (child.$children().length === 0 ? headerButtonRect.width : 0),
                },
            });
        }
        // get the gridlines that we do not need
        const gridLinesToRemove = this.existingGridLines.filter((line) => !gridLines.find((l) => l.id === line.id));
        if (this.redrawCounter !== currentRedrawCounter) {
            return;
        }
        for (const line of gridLinesToRemove) {
            line.element.remove();
        }
        if (this.redrawCounter !== currentRedrawCounter) {
            return;
        }
        // remove the gridlines from the existingGridLines array
        this.existingGridLines = this.existingGridLines.filter((line) => !gridLinesToRemove.find((l) => l.id === line.id));
        // add the new gridlines to the existingGridLines array
        for (const line of gridLines) {
            if (this.redrawCounter !== currentRedrawCounter) {
                return;
            }
            let l = this.existingGridLines.find((f) => f.id === line.id);
            if (!l) {
                l = {
                    id: line.id,
                    element: document.createElement('div'),
                };
                l.element.className = 'grid-line';
                l.element.style.position = 'absolute';
                this.selfElementRef.nativeElement.appendChild(l.element);
                this.existingGridLines.push(l);
            }
            l.element.style.top = `${line.from.top}px`;
            l.element.style.left = `${line.from.left}px`;
            l.element.style.width = `${line.to.left - line.from.left}px`;
            l.element.style.height = `${line.to.top - line.from.top}px`;
            l.element.style.pointerEvents = 'none';
            l.element.style.borderBottom = this.$gridLinesBorder();
            // only draw the left border if its the last child of the parent
            l.element.style.borderLeft = this.$gridLinesBorder();
            l.element.style.borderBottomLeftRadius = this.$gridLinesBorderRadius();
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TabTreeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: TabTreeComponent, isStandalone: false, selector: "tab-tree", inputs: { $showRootGridLines: { classPropertyName: "$showRootGridLines", publicName: "showRootGridLines", isSignal: true, isRequired: false, transformFunction: null }, $gridLinesBorder: { classPropertyName: "$gridLinesBorder", publicName: "gridLinesBorder", isSignal: true, isRequired: false, transformFunction: null }, $gridLinesBorderRadius: { classPropertyName: "$gridLinesBorderRadius", publicName: "gridLinesBorderRadius", isSignal: true, isRequired: false, transformFunction: null }, $expandButtonSize: { classPropertyName: "$expandButtonSize", publicName: "expandButtonSize", isSignal: true, isRequired: false, transformFunction: null }, $expandButtonColor: { classPropertyName: "$expandButtonColor", publicName: "expandButtonColor", isSignal: true, isRequired: false, transformFunction: null }, $expandButtonGap: { classPropertyName: "$expandButtonGap", publicName: "expandButtonGap", isSignal: true, isRequired: false, transformFunction: null }, $expandButtonTooltip: { classPropertyName: "$expandButtonTooltip", publicName: "expandButtonTooltip", isSignal: true, isRequired: false, transformFunction: null }, $expandButtonAlign: { classPropertyName: "$expandButtonAlign", publicName: "expandButtonAlign", isSignal: true, isRequired: false, transformFunction: null }, $childrenIndent: { classPropertyName: "$childrenIndent", publicName: "childrenIndent", isSignal: true, isRequired: false, transformFunction: null }, $keepButtonOffsetOnNoChildren: { classPropertyName: "$keepButtonOffsetOnNoChildren", publicName: "keepButtonOffsetOnNoChildren", isSignal: true, isRequired: false, transformFunction: null }, $hierarchyMode: { classPropertyName: "$hierarchyMode", publicName: "hierarchyMode", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: "@for (node of $childrenDisplayOrder(); let i = $index; track i) {\n    @if (!node.$parent() || node.$allParentsExpanded()) {\n        <ng-container *ngTemplateOutlet=\"node.$template()!; context: templateParams\" />\n    }\n}\n", styles: [":host{display:flex;flex-direction:column;position:relative}\n"], dependencies: [{ kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TabTreeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-tree', standalone: false, changeDetection: ChangeDetectionStrategy.OnPush, template: "@for (node of $childrenDisplayOrder(); let i = $index; track i) {\n    @if (!node.$parent() || node.$allParentsExpanded()) {\n        <ng-container *ngTemplateOutlet=\"node.$template()!; context: templateParams\" />\n    }\n}\n", styles: [":host{display:flex;flex-direction:column;position:relative}\n"] }]
        }], ctorParameters: () => [] });

class CollapsedContentDirective {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: CollapsedContentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "20.0.3", type: CollapsedContentDirective, isStandalone: false, selector: "[collapsedContent]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: CollapsedContentDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[collapsedContent]',
                    standalone: false,
                }]
        }] });

class ExpandedContentDirective {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: ExpandedContentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "20.0.3", type: ExpandedContentDirective, isStandalone: false, selector: "[expandedContent]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: ExpandedContentDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[expandedContent]',
                    standalone: false,
                }]
        }] });

class TabTreeNodeComponent {
    headerButtonId = generateRandomString(16);
    // used when hierarchyMode is auto to get direct parent
    hierarchyModeAutoParent = inject(TabTreeNodeComponent, {
        skipSelf: true,
        optional: true,
    });
    tabTree = inject(TabTreeComponent, {
        skipSelf: true,
        optional: true,
    });
    $expanded = model(false, {
        alias: 'expanded',
    });
    expanded$ = toObservable(this.$expanded);
    expandedChange = output();
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $collapsedContent = contentChild(CollapsedContentDirective);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $expandedContent = contentChild(ExpandedContentDirective);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $expandButtonColor = input(undefined, {
        alias: 'expandButtonColor',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $template = viewChild('treeNodeTemplate');
    $hierarchyId = input(generateRandomString(16), {
        alias: 'hierarchyId',
    });
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $hierarchyParentId = input(this.hierarchyModeAutoParent?.$hierarchyId(), {
        alias: 'hierarchyParentId',
    });
    $children = signal([]);
    // nullable Signal type needs to be set explicitly -> ng-packagr strips nullability
    $parent = signal(null);
    $allParentsExpanded = computed(() => {
        const parents = [];
        let parent = this.$parent();
        while (parent) {
            parents.push(parent);
            parent = parent.$parent();
        }
        return parents.every((p) => p.$expanded());
    });
    $depth = signal(-1);
    $order = input(0, {
        alias: 'order',
    });
    // The registry of the tree node. It is optional. If provided, the node will register itself and will be detected by the tree even if it's located in a sub-component
    $registry = input(undefined, {
        alias: 'registry',
    });
    ngOnInit() {
        const registry = this.$registry() ?? this.tabTree?.registry;
        if (!registry) {
            throw new Error('TreeNodeComponent: registry is not provided. Please provide a registry or use the TabTreeComponent.');
        }
        registry.register(this);
    }
    ngOnDestroy() {
        const registry = this.$registry() ?? this.tabTree?.registry;
        if (!registry) {
            console.warn('TreeNodeComponent: registry is not provided. Please provide a registry or use the TabTreeComponent.');
        }
        registry?.unregister(this);
    }
    setExpanded(expanded) {
        this.$expanded.set(expanded);
        this.expandedChange.emit(expanded);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TabTreeNodeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "20.0.3", type: TabTreeNodeComponent, isStandalone: false, selector: "tab-tree-node", inputs: { $expanded: { classPropertyName: "$expanded", publicName: "expanded", isSignal: true, isRequired: false, transformFunction: null }, $expandButtonColor: { classPropertyName: "$expandButtonColor", publicName: "expandButtonColor", isSignal: true, isRequired: false, transformFunction: null }, $hierarchyId: { classPropertyName: "$hierarchyId", publicName: "hierarchyId", isSignal: true, isRequired: false, transformFunction: null }, $hierarchyParentId: { classPropertyName: "$hierarchyParentId", publicName: "hierarchyParentId", isSignal: true, isRequired: false, transformFunction: null }, $order: { classPropertyName: "$order", publicName: "order", isSignal: true, isRequired: false, transformFunction: null }, $registry: { classPropertyName: "$registry", publicName: "registry", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { $expanded: "expandedChange", expandedChange: "expandedChange" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => TabTreeNodeComponent),
                multi: true,
            },
        ], queries: [{ propertyName: "$collapsedContent", first: true, predicate: CollapsedContentDirective, descendants: true, isSignal: true }, { propertyName: "$expandedContent", first: true, predicate: ExpandedContentDirective, descendants: true, isSignal: true }], viewQueries: [{ propertyName: "$template", first: true, predicate: ["treeNodeTemplate"], descendants: true, isSignal: true }], ngImport: i0, template: "<ng-template\n    #treeNodeTemplate\n    let-$childrenIndent=\"$childrenIndent\"\n    let-$expandButtonSize=\"$expandButtonSize\"\n    let-$expandButtonColorInherited=\"$expandButtonColor\"\n    let-$expandButtonGap=\"$expandButtonGap\"\n    let-$expandButtonTooltip=\"$expandButtonTooltip\"\n    let-$expandButtonAlign=\"$expandButtonAlign\"\n    let-$keepButtonOffsetOnNoChildren=\"$keepButtonOffsetOnNoChildren\"\n>\n    <div class=\"tree-node-container\" [style.marginLeft]=\"`calc(${$depth()} * ${$childrenIndent()})`\" [style.alignItems]=\"$expandButtonAlign()\" [style.gap]=\"$expandButtonGap()\">\n        <button\n            layout=\"icon\"\n            color=\"plain\"\n            class=\"header-button\"\n            type=\"button\"\n            [id]=\"headerButtonId\"\n            [inline]=\"true\"\n            [style.visibility]=\"$children().length === 0 ? 'hidden' : 'visible'\"\n            [style.fontSize]=\"$expandButtonSize()\"\n            [style.width]=\"$children().length !== 0 || $keepButtonOffsetOnNoChildren() ? $expandButtonSize() : '0px'\"\n            [style.minWidth]=\"'0px'\"\n            [style.height]=\"$expandButtonSize()\"\n            [style.color]=\"$expandButtonColor() || $expandButtonColorInherited()\"\n            [tooltip]=\"$expanded() ? $expandButtonTooltip()?.collapse : $expandButtonTooltip()?.expand\"\n            [tooltipPosition]=\"$expandButtonTooltip()?.position\"\n            [tooltipMargin]=\"$expandButtonTooltip()?.margin\"\n            [tooltipContext]=\"$expandButtonTooltip()?.context\"\n            (click)=\"setExpanded(!$expanded())\"\n        >\n            <tab-icon [value]=\"$expanded() ? 'keyboard_arrow_down' : 'keyboard_arrow_right'\" />\n        </button>\n        <div class=\"header-content\">\n            @if ($expanded() && $expandedContent()) {\n                <ng-content select=\"[expandedContent]\" />\n            } @else if (!$expanded() && $collapsedContent()) {\n                <ng-content select=\"[collapsedContent]\" />\n            }\n            <ng-content />\n        </div>\n    </div>\n</ng-template>\n", styles: [".tree-node-container{display:flex}.header-content{width:100%}\n"], dependencies: [{ kind: "component", type: i1$1.ButtonComponent, selector: "button:not([tab-menu-button]),a[button]:not([tab-menu-button])", inputs: ["disabled", "loading", "tabindex", "type", "kind", "color", "layout", "inline", "noFocusOnClick"] }, { kind: "component", type: i2.IconComponent, selector: "tab-icon", inputs: ["value", "color", "type", "fill", "weight", "grade", "opticalSizePx"] }, { kind: "directive", type: i3.TooltipDirective, selector: "[tooltip]", inputs: ["tooltip", "tooltipContext", "tooltipPosition", "tooltipMargin", "tooltipPadding", "tooltipArgsMode", "tooltipFullArgs", "tooltipLazyArgs"], outputs: ["tooltipChange", "tooltipContextChange", "tooltipPositionChange", "tooltipMarginChange", "tooltipPaddingChange"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TabTreeNodeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'tab-tree-node', standalone: false, providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => TabTreeNodeComponent),
                            multi: true,
                        },
                    ], changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-template\n    #treeNodeTemplate\n    let-$childrenIndent=\"$childrenIndent\"\n    let-$expandButtonSize=\"$expandButtonSize\"\n    let-$expandButtonColorInherited=\"$expandButtonColor\"\n    let-$expandButtonGap=\"$expandButtonGap\"\n    let-$expandButtonTooltip=\"$expandButtonTooltip\"\n    let-$expandButtonAlign=\"$expandButtonAlign\"\n    let-$keepButtonOffsetOnNoChildren=\"$keepButtonOffsetOnNoChildren\"\n>\n    <div class=\"tree-node-container\" [style.marginLeft]=\"`calc(${$depth()} * ${$childrenIndent()})`\" [style.alignItems]=\"$expandButtonAlign()\" [style.gap]=\"$expandButtonGap()\">\n        <button\n            layout=\"icon\"\n            color=\"plain\"\n            class=\"header-button\"\n            type=\"button\"\n            [id]=\"headerButtonId\"\n            [inline]=\"true\"\n            [style.visibility]=\"$children().length === 0 ? 'hidden' : 'visible'\"\n            [style.fontSize]=\"$expandButtonSize()\"\n            [style.width]=\"$children().length !== 0 || $keepButtonOffsetOnNoChildren() ? $expandButtonSize() : '0px'\"\n            [style.minWidth]=\"'0px'\"\n            [style.height]=\"$expandButtonSize()\"\n            [style.color]=\"$expandButtonColor() || $expandButtonColorInherited()\"\n            [tooltip]=\"$expanded() ? $expandButtonTooltip()?.collapse : $expandButtonTooltip()?.expand\"\n            [tooltipPosition]=\"$expandButtonTooltip()?.position\"\n            [tooltipMargin]=\"$expandButtonTooltip()?.margin\"\n            [tooltipContext]=\"$expandButtonTooltip()?.context\"\n            (click)=\"setExpanded(!$expanded())\"\n        >\n            <tab-icon [value]=\"$expanded() ? 'keyboard_arrow_down' : 'keyboard_arrow_right'\" />\n        </button>\n        <div class=\"header-content\">\n            @if ($expanded() && $expandedContent()) {\n                <ng-content select=\"[expandedContent]\" />\n            } @else if (!$expanded() && $collapsedContent()) {\n                <ng-content select=\"[collapsedContent]\" />\n            }\n            <ng-content />\n        </div>\n    </div>\n</ng-template>\n", styles: [".tree-node-container{display:flex}.header-content{width:100%}\n"] }]
        }] });

class TableauUiTreeModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiTreeModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.3", ngImport: i0, type: TableauUiTreeModule, declarations: [TabTreeComponent, TabTreeNodeComponent, ExpandedContentDirective, CollapsedContentDirective], imports: [CommonModule, TableauUiButtonModule, TableauUiIconModule, TableauUiTooltipModule], exports: [TabTreeComponent, TabTreeNodeComponent, ExpandedContentDirective, CollapsedContentDirective] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiTreeModule, imports: [CommonModule, TableauUiButtonModule, TableauUiIconModule, TableauUiTooltipModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.3", ngImport: i0, type: TableauUiTreeModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, TableauUiButtonModule, TableauUiIconModule, TableauUiTooltipModule],
                    declarations: [TabTreeComponent, TabTreeNodeComponent, ExpandedContentDirective, CollapsedContentDirective],
                    exports: [TabTreeComponent, TabTreeNodeComponent, ExpandedContentDirective, CollapsedContentDirective],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { CollapsedContentDirective, ExpandedContentDirective, TabTreeComponent, TabTreeNodeComponent, TableauUiTreeModule, TreeNodeRegistry };
//# sourceMappingURL=tableau-ui-angular-tree.mjs.map
